@@ -19,6 +19,8 @@
 #include "gserrors.h"
 #include "gsbitops.h"
 #include "gxdevice.h"
+#include "gxdcolor.h"		/* for gx_fill_rectangle_device_rop */
+#include "gxpcolor.h"           /* for gx_dc_devn_masked */
 #include "gxdevmem.h"           /* semi-public definitions */
 #include "gxgetbit.h"
 #include "gdevmem.h"            /* private definitions */
@@ -40,22 +42,16 @@ static dev_proc_get_bits_rectangle(mem_planar_get_bits_rectangle);
 static dev_proc_fill_rectangle_hl_color(mem_planar_fill_rectangle_hl_color);
 static dev_proc_put_image(mem_planar_put_image);
 
-/* It's a bit nasty to have to fork the planar dev_spec_op like this, but
- * the forwarding nature of the device means that the function pointer test
- * for the map_cmyk_color call fails if we let it fall through to the
- * default device. */
 static int
 mem_planar_dev_spec_op(gx_device *pdev, int dev_spec_op,
                        void *data, int size)
 {
     cmm_dev_profile_t *dev_profile;
 
-    if (dev_spec_op == gxdso_is_native_planar)
-        return 1;
     if (dev_spec_op == gxdso_supports_devn) {
         dev_proc(pdev, get_profile)(pdev, &dev_profile);
         if (dev_profile != NULL && dev_profile->supports_devn &&
-            pdev->procs.fill_rectangle_hl_color == mem_planar_fill_rectangle_hl_color) 
+            pdev->procs.fill_rectangle_hl_color == mem_planar_fill_rectangle_hl_color)
             return 1;
     }
     return gx_default_dev_spec_op(pdev, dev_spec_op, data, size);
@@ -65,8 +61,6 @@ static int
 mem_planar_dev_spec_op_cmyk4(gx_device *pdev, int dev_spec_op,
                              void *data, int size)
 {
-    if (dev_spec_op == gxdso_is_native_planar)
-        return 1;
     if (dev_spec_op == gxdso_is_std_cmyk_1bit)
         return 1;
     return gx_default_dev_spec_op(pdev, dev_spec_op, data, size);
@@ -92,7 +86,7 @@ gdev_mem_set_planar(gx_device_memory * mdev, int num_planes,
     int pi;
     const gx_device_memory *mdproto = gdev_mem_device_for_bits(mdev->color_info.depth);
 
-    if (num_planes < 1 || num_planes > GX_DEVICE_COLOR_MAX_COMPONENTS)
+    if (num_planes < 1 || num_planes > GX_DEVICE_COLOR_MAX_COMPONENTS || num_planes != mdev->color_info.num_components)
         return_error(gs_error_rangecheck);
     for (pi = 0, total_depth = 0; pi < num_planes; ++pi) {
         int shift = planes[pi].shift;
@@ -116,7 +110,7 @@ gdev_mem_set_planar(gx_device_memory * mdev, int num_planes,
     }
     if (total_depth > mdev->color_info.depth)
         return_error(gs_error_rangecheck);
-    mdev->num_planes = num_planes;
+    mdev->is_planar = 1;
     memcpy(mdev->planes, planes, num_planes * sizeof(planes[0]));
     mdev->plane_depth = same_depth;
     /* Change the drawing procedures. */
@@ -134,10 +128,10 @@ gdev_mem_set_planar(gx_device_memory * mdev, int num_planes,
     } else {
         /* If we are going out to a separation device or one that has more than
            four planes then use the high level color filling procedure.  Also
-           make use of the put_image operation to go from the pdf14 device 
+           make use of the put_image operation to go from the pdf14 device
            directly to the planar buffer. */
-        if (mdev->num_planes >= 4) {
-            set_dev_proc(mdev, fill_rectangle_hl_color, 
+        if (num_planes >= 4) {
+            set_dev_proc(mdev, fill_rectangle_hl_color,
                          mem_planar_fill_rectangle_hl_color);
             set_dev_proc(mdev, put_image, mem_planar_put_image);
         }
@@ -146,13 +140,13 @@ gdev_mem_set_planar(gx_device_memory * mdev, int num_planes,
         set_dev_proc(mdev, copy_mono, mem_planar_copy_mono);
         set_dev_proc(mdev, dev_spec_op, mem_planar_dev_spec_op);
         if ((mdev->color_info.depth == 24) &&
-            (mdev->num_planes == 3) &&
+            (num_planes == 3) &&
             (mdev->planes[0].depth == 8) && (mdev->planes[0].shift == 16) &&
             (mdev->planes[1].depth == 8) && (mdev->planes[1].shift == 8) &&
             (mdev->planes[2].depth == 8) && (mdev->planes[2].shift == 0))
             set_dev_proc(mdev, copy_color, mem_planar_copy_color_24to8);
         else if ((mdev->color_info.depth == 4) &&
-                 (mdev->num_planes == 4) &&
+                 (num_planes == 4) &&
                  (mdev->planes[0].depth == 1) && (mdev->planes[0].shift == 3) &&
                  (mdev->planes[1].depth == 1) && (mdev->planes[1].shift == 2) &&
                  (mdev->planes[2].depth == 1) && (mdev->planes[2].shift == 1) &&
@@ -162,13 +156,13 @@ gdev_mem_set_planar(gx_device_memory * mdev, int num_planes,
         } else
             set_dev_proc(mdev, copy_color, mem_planar_copy_color);
         set_dev_proc(mdev, copy_alpha, gx_default_copy_alpha);
-        set_dev_proc(mdev, copy_planes, mem_planar_copy_planes);
         set_dev_proc(mdev, strip_tile_rectangle, mem_planar_strip_tile_rectangle);
         set_dev_proc(mdev, strip_tile_rect_devn, mem_planar_strip_tile_rect_devn);
         set_dev_proc(mdev, strip_copy_rop, mem_planar_strip_copy_rop);
         set_dev_proc(mdev, strip_copy_rop2, mem_planar_strip_copy_rop2);
         set_dev_proc(mdev, get_bits_rectangle, mem_planar_get_bits_rectangle);
     }
+    set_dev_proc(mdev, copy_planes, mem_planar_copy_planes);
     return 0;
 }
 
@@ -179,7 +173,7 @@ mem_planar_open(gx_device * dev)
     gx_device_memory *const mdev = (gx_device_memory *)dev;
 
     /* Check that we aren't trying to open a chunky device as planar. */
-    if (mdev->num_planes == 0)
+    if (!dev->is_planar)
         return_error(gs_error_rangecheck);
     return gdev_mem_open_scan_lines(mdev, dev->height);
 }
@@ -205,7 +199,9 @@ typedef struct mem_save_params_s {
  * at least 2 line pointers to use. Otherwise, we fall back to the old
  * method.
  */
-/* FIXME: Find a nicer way of calculating raster. */
+/* FIXME: Find a nicer way of calculating raster. This is only required if
+ * we allow the plane_depth to vary per plane, and the rest of the code
+ * assumes that it never does. This can probably be simplified now. */
 #define MEM_SET_PARAMS(mdev, plane_depth)\
   (mdev->color_info.depth = plane_depth, /* maybe not needed */\
    mdev->base = mdev->line_ptrs[0],\
@@ -215,53 +211,95 @@ typedef struct mem_save_params_s {
    mdev->base = msp.base,\
    mdev->line_ptrs = msp.line_ptrs)
 
+static int
+put_image_copy_planes(gx_device * dev, const byte **base_ptr, int sourcex,
+                      int sraster, gx_bitmap_id id,
+                      int x, int y, int w, int h)
+{
+    gx_device_memory * const mdev = (gx_device_memory *)dev;
+    int plane_depth;
+    mem_save_params_t save;
+    const gx_device_memory *mdproto;
+    int code = 0;
+    uchar plane;
+
+    MEM_SAVE_PARAMS(mdev, save);
+    for (plane = 0; plane < mdev->color_info.num_components; plane++)
+    {
+        const byte *base = base_ptr[plane];
+        plane_depth = mdev->planes[plane].depth;
+        mdproto = gdev_mem_device_for_bits(plane_depth);
+        if (base == NULL) {
+            /* Blank the plane */
+            code = dev_proc(mdproto, fill_rectangle)(dev, x, y, w, h,
+                (gx_color_index)(dev->color_info.polarity == GX_CINFO_POLARITY_ADDITIVE ? 0 : -1));
+        } else if (plane_depth == 1)
+            code = dev_proc(mdproto, copy_mono)(dev, base, sourcex, sraster, id,
+                                                x, y, w, h,
+                                                (gx_color_index)0,
+                                                (gx_color_index)1);
+        else
+            code = dev_proc(mdproto, copy_color)(dev, base, sourcex, sraster,
+                                                 id, x, y, w, h);
+        mdev->line_ptrs += mdev->height;
+    }
+    MEM_RESTORE_PARAMS(mdev, save);
+    return code;
+}
+
 /* Put image command for copying the planar image buffers with or without
    alpha directly to the device buffer */
 static int
-mem_planar_put_image(gx_device *pdev, const byte *buffer, int num_chan, int xstart,
+mem_planar_put_image(gx_device *pdev, const byte **buffers, int num_chan, int xstart,
               int ystart, int width, int height, int row_stride,
-              int plane_stride, int alpha_plane_index, int tag_plane_index)
+              int alpha_plane_index, int tag_plane_index)
 {
     gx_device_memory * const mdev = (gx_device_memory *)pdev;
 
-    /* We don't want alpha, return 0 to ask for the pdf14 device to do the 
-       alpha composition. We also do not want chunky data coming in or to deal 
-       with planar devices that are not 8 bit per colorant */    
-    if (alpha_plane_index != 0 || plane_stride == 0 || 
+    /* We don't want alpha, return 0 to ask for the pdf14 device to do the
+       alpha composition. We also do not want chunky data coming in or to deal
+       with planar devices that are not 8 bit per colorant */
+    if (alpha_plane_index != 0 ||
         mdev->planes[0].depth != 8)
-        return 0;  
+        return 0;
 
-    (*dev_proc(pdev, copy_planes)) (pdev, buffer, 0, row_stride,
-                                 gx_no_bitmap_id, xstart, ystart,
-                                 width, height, plane_stride/row_stride);
+    put_image_copy_planes(pdev, buffers, 0, row_stride,
+                          gx_no_bitmap_id, xstart, ystart,
+                          width, height);
     /* we used all of the data */
-    return height;        
+    return height;
 }
 
-/* Fill a rectangle with a high level color.  This is used for separation 
+/* Fill a rectangle with a high level color.  This is used for separation
    devices. (e.g. tiffsep, psdcmyk) */
 static int
 mem_planar_fill_rectangle_hl_color(gx_device *dev, const gs_fixed_rect *rect,
-    const gs_imager_state *pis, const gx_drawing_color *pdcolor,
+    const gs_gstate *pgs, const gx_drawing_color *pdcolor,
     const gx_clip_path *pcpath)
 {
     gx_device_memory * const mdev = (gx_device_memory *)dev;
     mem_save_params_t save;
-    int pi;
+    uchar pi;
+    int x = fixed2int(rect->p.x);
+    int y = fixed2int(rect->p.y);
+    int w = fixed2int(rect->q.x) - x;
+    int h = fixed2int(rect->q.y) - y;
 
+    /* We can only handle devn cases, so use the default if not */
+    /* We can get called here from gx_dc_devn_masked_fill_rectangle */
+    if (pdcolor->type != gx_dc_type_devn && pdcolor->type != &gx_dc_devn_masked) {
+        return gx_fill_rectangle_device_rop( x, y, w, h, pdcolor, dev, lop_default);
+    }
     MEM_SAVE_PARAMS(mdev, save);
-    for (pi = 0; pi < mdev->num_planes; ++pi) {
+    for (pi = 0; pi < mdev->color_info.num_components; ++pi) {
         int plane_depth = mdev->planes[pi].depth;
         gx_color_index mask = ((gx_color_index)1 << plane_depth) - 1;
         int shift = 16 - plane_depth;
         const gx_device_memory *mdproto = gdev_mem_device_for_bits(plane_depth);
 
         MEM_SET_PARAMS(mdev, plane_depth);
-        dev_proc(mdproto, fill_rectangle)(dev, rect->p.x, rect->p.y, 
-                                          rect->q.x - rect->p.x, 
-                                          rect->q.y - rect->p.y,
-                                          (pdcolor->colors.devn.values[pi]) >>
-                                          shift & mask); 
+        dev_proc(mdproto, fill_rectangle)(dev, x, y, w, h,
+                                          (pdcolor->colors.devn.values[pi]) >> shift & mask);
         mdev->line_ptrs += mdev->height;
     }
     MEM_RESTORE_PARAMS(mdev, save);
@@ -275,10 +313,10 @@ mem_planar_fill_rectangle(gx_device * dev, int x, int y, int w, int h,
 {
     gx_device_memory * const mdev = (gx_device_memory *)dev;
     mem_save_params_t save;
-    int pi;
+    uchar pi;
 
     MEM_SAVE_PARAMS(mdev, save);
-    for (pi = 0; pi < mdev->num_planes; ++pi) {
+    for (pi = 0; pi < mdev->color_info.num_components; ++pi) {
         int plane_depth = mdev->planes[pi].depth;
         gx_color_index mask = ((gx_color_index)1 << plane_depth) - 1;
         const gx_device_memory *mdproto =
@@ -302,10 +340,10 @@ mem_planar_copy_mono(gx_device * dev, const byte * base, int sourcex,
 {
     gx_device_memory * const mdev = (gx_device_memory *)dev;
     mem_save_params_t save;
-    int pi;
+    uchar pi;
 
     MEM_SAVE_PARAMS(mdev, save);
-    for (pi = 0; pi < mdev->num_planes; ++pi) {
+    for (pi = 0; pi < mdev->color_info.num_components; ++pi) {
         int plane_depth = mdev->planes[pi].depth;
         int shift = mdev->planes[pi].shift;
         gx_color_index mask = ((gx_color_index)1 << plane_depth) - 1;
@@ -810,11 +848,11 @@ mem_planar_copy_color(gx_device * dev, const byte * base, int sourcex,
     } buf;
     int source_depth = dev->color_info.depth;
     mem_save_params_t save;
-    int pi;
+    uchar pi;
 
     fit_copy(dev, base, sourcex, sraster, id, x, y, w, h);
     MEM_SAVE_PARAMS(mdev, save);
-    for (pi = 0; pi < mdev->num_planes; ++pi) {
+    for (pi = 0; pi < mdev->color_info.num_components; ++pi) {
         int plane_depth = mdev->planes[pi].depth;
         int shift = mdev->planes[pi].shift;
         gx_color_index mask = ((gx_color_index)1 << plane_depth) - 1;
@@ -860,21 +898,29 @@ mem_planar_copy_color(gx_device * dev, const byte * base, int sourcex,
                     source_bit = xbit & 7;
                 }
                 for (iy = 0; iy < ch; ++iy) {
-                    sample_load_declare_setup(sptr, sbit, source_base,
-                                              source_bit, source_depth);
-                    sample_store_declare_setup(dptr, dbit, dbbyte,
-                                               buf.b + br * iy,
-                                               0, plane_depth);
+                    const byte *sptr =source_base;
+                    int sbit = source_bit;
+                    byte *dptr = buf.b + br * iy;
+                    int dbit = 0;
+                    byte dbbyte = (dbit ? (byte)(*dptr & (0xff00 >> dbit)) : 0);
 
                     for (ix = 0; ix < cw; ++ix) {
                         gx_color_index value;
 
-                        sample_load_next_any(value, sptr, sbit, source_depth);
+                        if (sizeof(value) > 4){
+                            if (sample_load_next64((uint64_t *)&value, &sptr, &sbit, source_depth) < 0)
+                                return_error(gs_error_rangecheck);
+                        }
+                        else {
+                            if (sample_load_next32((uint32_t *)&value, &sptr, &sbit, source_depth) < 0)
+                                return_error(gs_error_rangecheck);
+                        }
                         value = (value >> shift) & mask;
-                        sample_store_next16(value, dptr, dbit, plane_depth,
-                                            dbbyte);
+                        if (sample_store_next16(value, &dptr, &dbit, plane_depth,
+                                            &dbbyte) < 0)
+                            return_error(gs_error_rangecheck);
                     }
-                    sample_store_flush(dptr, dbit, plane_depth, dbbyte);
+                    sample_store_flush(dptr, dbit, dbbyte);
                     source_base += sraster;
                 }
                 /*
@@ -909,10 +955,10 @@ mem_planar_copy_planes(gx_device * dev, const byte * base, int sourcex,
     mem_save_params_t save;
     const gx_device_memory *mdproto;
     int code = 0;
-    int plane;
+    uchar plane;
 
     MEM_SAVE_PARAMS(mdev, save);
-    for (plane = 0; plane < mdev->num_planes; plane++)
+    for (plane = 0; plane < mdev->color_info.num_components; plane++)
     {
         plane_depth = mdev->planes[plane].depth;
         mdproto = gdev_mem_device_for_bits(plane_depth);
@@ -934,32 +980,37 @@ mem_planar_copy_planes(gx_device * dev, const byte * base, int sourcex,
 int
 mem_planar_strip_tile_rect_devn(gx_device * dev, const gx_strip_bitmap * tiles,
                                 int x, int y, int w, int h,
-                                const gx_drawing_color *pdcolor0, 
+                                const gx_drawing_color *pdcolor0,
                                 const gx_drawing_color *pdcolor1, int px, int py)
 {
     gx_device_memory * const mdev = (gx_device_memory *)dev;
     mem_save_params_t save;
-    int pi;
+    uchar pi;
 
     MEM_SAVE_PARAMS(mdev, save);
-    for (pi = 0; pi < mdev->num_planes; ++pi) {
+    for (pi = 0; pi < mdev->color_info.num_components; ++pi) {
         int plane_depth = mdev->planes[pi].depth;
         gx_color_index mask = ((gx_color_index)1 << plane_depth) - 1;
         int shift = 16 - plane_depth;
         const gx_device_memory *mdproto =
             gdev_mem_device_for_bits(plane_depth);
         gx_color_index c1, c0;
-        
-        if (pdcolor0->type == gx_dc_type_pure) {
-            c0 = gx_no_color_index;
-        } else {
+
+        if (pdcolor0->type == gx_dc_type_devn) {
             c0 = (pdcolor0->colors.devn.values[pi]) >> shift & mask;
-        }
-        if (pdcolor1->type == gx_dc_type_pure) {
-            c1 = gx_no_color_index;
         } else {
-            c1 = (pdcolor1->colors.devn.values[pi]) >> shift & mask;
+            c0 = gx_no_color_index;
         }
+        if (pdcolor1->type == gx_dc_type_devn) {
+            c1 = (pdcolor1->colors.devn.values[pi]) >> shift & mask;
+        } else {
+            c1 = gx_no_color_index;
+        }
+#ifdef DEBUG
+        if (c0 == gx_no_color_index && c1 == gx_no_color_index) {
+            dprintf("mem_planar_strip_tile_rect_dev called with two non-devn colors\n");
+        }
+#endif
         MEM_SET_PARAMS(mdev, plane_depth);
         if (c0 == c1)
             dev_proc(mdproto, fill_rectangle)(dev, x, y, w, h, c0);
@@ -987,14 +1038,14 @@ mem_planar_strip_tile_rectangle(gx_device * dev, const gx_strip_bitmap * tiles,
 {
     gx_device_memory * const mdev = (gx_device_memory *)dev;
     mem_save_params_t save;
-    int pi;
+    uchar pi;
 
     /* We can't split up the transfer if the tile is colored. */
     if (color0 == gx_no_color_index && color1 == gx_no_color_index)
         return gx_default_strip_tile_rectangle
             (dev, tiles, x, y, w, h, color0, color1, px, py);
     MEM_SAVE_PARAMS(mdev, save);
-    for (pi = 0; pi < mdev->num_planes; ++pi) {
+    for (pi = 0; pi < mdev->color_info.num_components; ++pi) {
         int plane_depth = mdev->planes[pi].depth;
         int shift = mdev->planes[pi].shift;
         gx_color_index mask = ((gx_color_index)1 << plane_depth) - 1;
@@ -1043,8 +1094,8 @@ planar_cmyk4bit_strip_copy_rop(gx_device_memory * mdev,
     byte lmask, rmask;
     rop_proc cproc, mproc, yproc;
     int dbit;
-    int cscolor, mscolor, yscolor, kscolor;
-    int ctcolor, mtcolor, ytcolor, ktcolor;
+    int cscolor = 0, mscolor = 0, yscolor = 0, kscolor = 0;
+    int ctcolor = 0, mtcolor = 0, ytcolor = 0, ktcolor = 0;
     int constant_s = 0;
 
     /* Modify the raster operation according to the source palette. */
@@ -1135,7 +1186,9 @@ planar_cmyk4bit_strip_copy_rop(gx_device_memory * mdev,
         mdrow = cdrow + mdev->height * draster;
         ydrow = mdrow + mdev->height * draster;
         kdrow = ydrow + mdev->height * draster;
-        traster = (textures ? textures->raster : 0);
+        if (!textures)
+            return 0;
+        traster = textures->raster;
         ty = y + phase_y;
         for (; line_count-- > 0; cdrow += draster, mdrow += draster, ydrow += draster, kdrow += draster, srow += sraster, ++ty) {
             int sx = sourcex;
@@ -1678,10 +1731,12 @@ planar_to_chunky(gx_device_memory *mdev, int x, int y, int w, int h,
                  int offset, uint draster, byte *dest, byte **line_ptrs,
                  int plane_height)
 {
-    int num_planes = mdev->num_planes;
-    sample_load_declare(sptr[GX_DEVICE_COLOR_MAX_COMPONENTS],
-                        sbit[GX_DEVICE_COLOR_MAX_COMPONENTS]);
-    sample_store_declare(dptr, dbit, dbbyte);
+    int num_planes = mdev->color_info.num_components;
+    const byte *sptr[GX_DEVICE_COLOR_MAX_COMPONENTS];
+    int sbit[GX_DEVICE_COLOR_MAX_COMPONENTS];
+    byte *dptr;
+    int dbit;
+    byte dbbyte;
     int ddepth = mdev->color_info.depth;
     int direct =
         (mdev->color_info.depth != num_planes * mdev->plane_depth ? 0 :
@@ -1709,13 +1764,13 @@ planar_to_chunky(gx_device_memory *mdev, int x, int y, int w, int h,
             int xbit = x * plane_depth;
 
             sptr[pi] = *line_ptr + (xbit >> 3);
-            sample_load_setup(sbit[pi], xbit & 7, plane_depth);
+            sbit[pi] = xbit & 7;
         }
         {
             int xbit = offset * ddepth;
 
             dptr = dest + (iy - y) * draster + (xbit >> 3);
-            sample_store_setup(dbit, xbit & 7, ddepth);
+            dbit = xbit & 7;
         }
         if (direct == -8) {
             /* 1 byte per component, lsb first. */
@@ -1744,20 +1799,29 @@ planar_to_chunky(gx_device_memory *mdev, int x, int y, int w, int h,
                 break;
             }
         }
-        sample_store_preload(dbbyte, dptr, dbit, ddepth);
+        dbbyte = (dbit ? (byte)(*dptr & (0xff00 >> dbit)) : 0);
+/*        sample_store_preload(dbbyte, dptr, dbit, ddepth);*/
         for (ix = w; ix > 0; --ix) {
             gx_color_index color = 0;
 
             for (pi = 0; pi < num_planes; ++pi) {
                 int plane_depth = mdev->planes[pi].depth;
-                uint value;
+                ushort value;
 
-                sample_load_next16(value, sptr[pi], sbit[pi], plane_depth);
+                if (sample_load_next16(&value, &sptr[pi], &sbit[pi], plane_depth) < 0)
+                    return_error(gs_error_rangecheck);
                 color |= (gx_color_index)value << mdev->planes[pi].shift;
             }
-            sample_store_next_any(color, dptr, dbit, ddepth, dbbyte);
+            if (sizeof(color) > 4) {
+                if (sample_store_next64(color, &dptr, &dbit, ddepth, &dbbyte) < 0)
+                    return_error(gs_error_rangecheck);
+            }
+            else {
+                if (sample_store_next32(color, &dptr, &dbit, ddepth, &dbbyte) < 0)
+                    return_error(gs_error_rangecheck);
+            }
         }
-        sample_store_flush(dptr, dbit, ddepth, dbbyte);
+        sample_store_flush(dptr, dbit, dbbyte);
     }
     return 0;
 }
@@ -1820,32 +1884,33 @@ mem_planar_strip_copy_rop2(gx_device * dev,
          * pointers for the buffer. For now, for the sake of sanity, we
          * convert whole lines of s, but only as many lines as we have to. */
         /* We assume that scolors == NULL here */
-        int i, j;
+        int i;
+        uchar j;
         uint chunky_sraster;
         uint nbytes;
         byte **line_ptrs;
         byte *sbuf, *buf;
 
-        chunky_sraster = sraster * mdev->num_planes;
+        chunky_sraster = sraster * mdev->color_info.num_components;
         nbytes = height * chunky_sraster;
         buf = gs_alloc_bytes(mdev->memory, nbytes, "mem_planar_strip_copy_rop(buf)");
         if (buf == NULL) {
             return gs_note_error(gs_error_VMerror);
         }
-        nbytes = sizeof(byte *) * mdev->num_planes * height;
+        nbytes = sizeof(byte *) * mdev->color_info.num_components * height;
         line_ptrs = (byte **)gs_alloc_bytes(mdev->memory, nbytes, "mem_planar_strip_copy_rop(line_ptrs)");
         if (line_ptrs == NULL) {
             gs_free_object(mdev->memory, buf, "mem_planar_strip_copy_rop(buf)");
             return gs_note_error(gs_error_VMerror);
         }
-        for (j = 0; j < mdev->num_planes; j++) {
-            sbuf = sdata + j * sraster;
+        for (j = 0; j < mdev->color_info.num_components; j++) {
+            sbuf = (byte *)sdata + j * sraster;
             for (i = height; i > 0; i--) {
                 *line_ptrs++ = sbuf;
                 sbuf += sraster;
             }
         }
-        line_ptrs -= height * mdev->num_planes;
+        line_ptrs -= height * mdev->color_info.num_components;
         planar_to_chunky(mdev, sourcex, 0, width, height,
                          0, chunky_sraster, buf, line_ptrs, planar_height);
         gs_free_object(mdev->memory, line_ptrs, "mem_planar_strip_copy_rop(line_ptrs)");
@@ -1890,18 +1955,18 @@ mem_planar_strip_copy_rop2(gx_device * dev,
         if (buf == NULL) {
             return gs_note_error(gs_error_VMerror);
         }
-        nbytes = sizeof(byte *) * mdev->num_planes * textures->rep_height;
+        nbytes = sizeof(byte *) * mdev->color_info.num_components * textures->rep_height;
         line_ptrs = (byte **)gs_alloc_bytes(mdev->memory, nbytes, "mem_planar_strip_copy_rop(line_ptrs)");
         if (line_ptrs == NULL) {
             gs_free_object(mdev->memory, buf, "mem_planar_strip_copy_rop(buf)");
             return gs_note_error(gs_error_VMerror);
         }
         tbuf = textures->data;
-        for (i = textures->rep_height * mdev->num_planes; i > 0; i--) {
+        for (i = textures->rep_height * mdev->color_info.num_components; i > 0; i--) {
             *line_ptrs++ = tbuf;
             tbuf += textures->raster;
         }
-        line_ptrs -= textures->rep_height * mdev->num_planes;
+        line_ptrs -= textures->rep_height * mdev->color_info.num_components;
         planar_to_chunky(mdev, 0, ty, textures->rep_width, chunky_t_height,
                          0, chunky_t_raster, buf, line_ptrs, textures->rep_height);
         gs_free_object(mdev->memory, line_ptrs, "mem_planar_strip_copy_rop(line_ptrs)");
@@ -1925,11 +1990,11 @@ mem_planar_strip_copy_rop2(gx_device * dev,
     if (!lop_uses_T(lop) || (tcolors && (tcolors[0] == tcolors[1]))) {
         /* No T in use, or constant T. */
         if ((!lop_uses_S(lop) || (scolors && (scolors[0] == scolors[1]))) &&
-            ((mdev->num_planes == 1) || (mdev->num_planes == 3))) {
-            int plane;
+            ((mdev->color_info.num_components == 1) || (mdev->color_info.num_components == 3))) {
+            uchar plane;
             /* No S in use, or constant S. And either greyscale or rgb,
              * so we can just do the rop on each plane in turn. */
-            for (plane=0; plane < mdev->num_planes; plane++)
+            for (plane=0; plane < mdev->color_info.num_components; plane++)
             {
                 gx_color_index tcolors2[2], scolors2[2];
                 int shift = mdev->planes[plane].shift;
@@ -1953,7 +2018,7 @@ mem_planar_strip_copy_rop2(gx_device * dev,
             }
             return 0;
         }
-        if ((mdev->num_planes == 4) && (mdev->plane_depth == 1) &&
+        if ((mdev->color_info.num_components == 4) && (mdev->plane_depth == 1) &&
             ((lop & (lop_S_transparent | lop_T_transparent)) == 0))
         {
             lop = cmykrop[lop & 0xff] | (lop & ~0xff);
@@ -1966,7 +2031,7 @@ mem_planar_strip_copy_rop2(gx_device * dev,
         }
     }
     if (!tcolors && !scolors &&
-        (mdev->num_planes == 4) && (mdev->plane_depth == 1) &&
+        (mdev->color_info.num_components == 4) && (mdev->plane_depth == 1) &&
         ((lop & (lop_S_transparent | lop_T_transparent)) == 0)) {
         lop = cmykrop[lop & 0xff] | (lop & ~0xff);
         return planar_cmyk4bit_strip_copy_rop(mdev, sdata, sourcex,
@@ -1990,11 +2055,11 @@ mem_planar_get_bits_rectangle(gx_device * dev, const gs_int_rect * prect,
                               gs_get_bits_params_t * params,
                               gs_int_rect ** unread)
 {
-    /* This duplicates most of mem_get_bits_rectangle.  Tant pis. */
+    /* This duplicates most of mem_get_bits_rectangle.  Tant pgs. */
     gx_device_memory * const mdev = (gx_device_memory *)dev;
     gs_get_bits_options_t options = params->options;
     int x = prect->p.x, w = prect->q.x - x, y = prect->p.y, h = prect->q.y - y;
-    int num_planes = mdev->num_planes;
+    uchar num_planes = mdev->color_info.num_components;
     gs_get_bits_params_t copy_params;
     int code;
 
@@ -2055,13 +2120,13 @@ mem_planar_get_bits_rectangle(gx_device * dev, const gs_int_rect * prect,
      */
     if (!(~options & (GB_PACKING_PLANAR | GB_SELECT_PLANES))) {
         /* Check that only a single plane is being requested. */
-        int pi;
+        uchar pi;
 
         for (pi = 0; pi < num_planes; ++pi)
             if (params->data[pi] != 0)
                 break;
         if (pi < num_planes) {
-            int plane = pi++;
+            uchar plane = pi++;
 
             for (; pi < num_planes; ++pi)
                 if (params->data[pi] != 0)
