@@ -614,6 +614,7 @@ gx_device_init(gx_device * dev, const gx_device * proto, gs_memory_t * mem,
     dev->memory = mem;
     dev->retained = !internal;
     rc_init(dev, mem, (internal ? 0 : 1));
+    rc_increment(dev->icc_struct);
 }
 
 void
@@ -826,8 +827,8 @@ gx_device_set_margins(gx_device * dev, const float *margins /*[4] */ ,
     for (i = 0; i < 4; ++i)
         dev->HWMargins[i] = margins[i] * 72.0;
     if (move_origin) {
-        dev->Margins[0] = -margins[0] * dev->MarginsHWResolution[0];
-        dev->Margins[1] = -margins[3] * dev->MarginsHWResolution[1];
+        dev->Margins[0] = -margins[0] * dev->HWResolution[0];
+        dev->Margins[1] = -margins[3] * dev->HWResolution[1];
     }
 }
 
@@ -960,7 +961,6 @@ gx_device_copy_params(gx_device *dev, const gx_device *target)
         COPY_ARRAY_PARAM(ImagingBBox);
         COPY_PARAM(ImagingBBox_set);
         COPY_ARRAY_PARAM(HWResolution);
-        COPY_ARRAY_PARAM(MarginsHWResolution);
         COPY_ARRAY_PARAM(Margins);
         COPY_ARRAY_PARAM(HWMargins);
         COPY_PARAM(PageCount);
@@ -1274,3 +1274,38 @@ gx_device_close_output_file(const gx_device * dev, const char *fname,
     gp_close_printer(dev->memory, file, (parsed.fname ? parsed.fname : fname));
     return 0;
 }
+
+bool gx_color_info_equal(const gx_device_color_info * p1, const gx_device_color_info * p2)
+{
+    if (p1->anti_alias.graphics_bits != p2->anti_alias.graphics_bits)
+        return false;
+    if (p1->anti_alias.text_bits != p2->anti_alias.text_bits)
+        return false;
+    if (p1->black_component != p2->black_component)
+        return false;
+    if (strcmp(p1->cm_name, p2->cm_name) != 0)
+        return false;
+    if (p1->depth != p2->depth)
+        return false;
+    if (p1->dither_colors != p2->dither_colors)
+        return false;
+    if (p1->dither_grays != p2->dither_grays)
+        return false;
+    if (p1->gray_index != p2->gray_index)
+        return false;
+    if (p1->max_color != p2->max_color)
+        return false;
+    if (p1->max_components != p2->max_components)
+        return false;
+    if (p1->opmode != p2->opmode)
+        return false;
+    if (p1->polarity != p2->polarity)
+        return false;
+    if (p1->process_comps != p2->process_comps)
+        return false;
+    if (p1->separable_and_linear != p2->separable_and_linear)
+        return false;
+    if (p1->use_antidropout_downscaler != p2->use_antidropout_downscaler)
+        return false;
+    return true;
+}
