@@ -43,7 +43,7 @@ gs_get_device_or_hw_params(gx_device * orig_dev, gs_param_list * plist,
      * prototype.
      */
     gx_device *dev;
-    int code;
+    int code = 0;
 
     if (orig_dev->memory)
         dev = orig_dev;
@@ -56,9 +56,13 @@ gs_get_device_or_hw_params(gx_device * orig_dev, gs_param_list * plist,
     fill_dev_proc(dev, get_params, gx_default_get_params);
     fill_dev_proc(dev, get_page_device, gx_default_get_page_device);
     fill_dev_proc(dev, get_alpha_bits, gx_default_get_alpha_bits);
-    code = (is_hardware ?
-            (*dev_proc(dev, get_hardware_params)) (dev, plist) :
-            (*dev_proc(dev, get_params)) (dev, plist));
+    if (is_hardware) {
+        if (dev_proc(dev, get_hardware_params) != NULL)
+            code = (*dev_proc(dev, get_hardware_params)) (dev, plist);
+    } else {
+        if (dev_proc(dev, get_params) != NULL)
+            code = (*dev_proc(dev, get_params)) (dev, plist);
+    }
     if (dev != orig_dev)
         gx_device_retain(dev, false);  /* frees the copy */
     return code;
@@ -69,7 +73,7 @@ int gx_default_get_param(gx_device *dev, char *Param, void *list)
     gs_param_list * plist = (gs_param_list *)list;
     int k, colors = dev->color_info.num_components;
     gs_param_string profile_array[NUM_DEVICE_PROFILES];
-    gs_param_string postren_profile;
+    gs_param_string postren_profile, blend_profile;
     gs_param_string proof_profile, link_profile, icc_colorants;
     gsicc_rendering_intents_t profile_intents[NUM_DEVICE_PROFILES];
     gsicc_blackptcomp_t blackptcomps[NUM_DEVICE_PROFILES];
@@ -168,11 +172,6 @@ int gx_default_get_param(gx_device *dev, char *Param, void *list)
         set_param_array(hwma, dev->HWMargins, 4);
         return param_write_float_array(plist, ".HWMargins", &hwma);
     }
-    if (strcmp(Param, ".MarginsHWResolution") == 0) {
-        gs_param_float_array mhwra;
-        set_param_array(mhwra, dev->MarginsHWResolution, 2);
-        return param_write_float_array(plist, ".MarginsHWResolution", &mhwra);
-    }
     if (strcmp(Param, ".MediaSize") == 0) {
         gs_param_float_array msa;
         set_param_array(msa, dev->MediaSize, 2);
@@ -304,6 +303,12 @@ int gx_default_get_param(gx_device *dev, char *Param, void *list)
                 blackpreserve[k] = dev_profile->rendercond[k].preserve_black;
             }
         }
+        if (dev_profile->blend_profile == NULL) {
+            param_string_from_string(blend_profile, null_str);
+        } else {
+            param_string_from_transient_string(blend_profile,
+                dev_profile->blend_profile->name);
+        }
         if (dev_profile->postren_profile == NULL) {
             param_string_from_string(postren_profile, null_str);
         } else {
@@ -350,6 +355,7 @@ int gx_default_get_param(gx_device *dev, char *Param, void *list)
             blackptcomps[k] = gsBPNOTSPECIFIED;
             blackpreserve[k] = gsBKPRESNOTSPECIFIED;
         }
+        param_string_from_string(blend_profile, null_str);
         param_string_from_string(postren_profile, null_str);
         param_string_from_string(proof_profile, null_str);
         param_string_from_string(link_profile, null_str);
@@ -369,10 +375,13 @@ int gx_default_get_param(gx_device *dev, char *Param, void *list)
     }
     if (strcmp(Param, "PreBandThreshold") == 0) {
         return param_write_bool(plist, "PreBandThreshold", &prebandthreshold);
-    }  
+    }
     if (strcmp(Param, "PostRenderProfile") == 0) {
         return param_write_string(plist, "PostRenderProfile", &(postren_profile));
     }
+    if (strcmp(Param, "BlendColorProfile") == 0) {
+        return param_write_string(plist, "BlendColorProfile", &(blend_profile));
+    }
     if (strcmp(Param, "ProofProfile") == 0) {
         return param_write_string(plist,"ProofProfile", &(proof_profile));
     }
@@ -476,7 +485,7 @@ gx_default_get_params(gx_device * dev, gs_param_list * plist)
 
     bool seprs = false;
     gs_param_string dns, pcms, profile_array[NUM_DEVICE_PROFILES];
-    gs_param_string postren_profile, pagelist;
+    gs_param_string blend_profile, postren_profile, pagelist;
     gs_param_string proof_profile, link_profile, icc_colorants;
     gsicc_rendering_intents_t profile_intents[NUM_DEVICE_PROFILES];
     gsicc_blackptcomp_t blackptcomps[NUM_DEVICE_PROFILES];
@@ -501,7 +510,7 @@ gx_default_get_params(gx_device * dev, gs_param_list * plist)
     int GrayValues = dev->color_info.max_gray + 1;
     int HWSize[2];
     gs_param_int_array hwsa;
-    gs_param_float_array hwma, mhwra;
+    gs_param_float_array hwma;
     cmm_dev_profile_t *dev_profile;
 
     /* Fill in page device parameters. */
@@ -529,7 +538,6 @@ gx_default_get_params(gx_device * dev, gs_param_list * plist)
     HWSize[1] = dev->height;
     set_param_array(hwsa, HWSize, 2);
     set_param_array(hwma, dev->HWMargins, 4);
-    set_param_array(mhwra, dev->MarginsHWResolution, 2);
     /* Check if the device profile is null.  If it is, then we need to
        go ahead and get it set up at this time.  If the proc is not
        set up yet then we are not going to do anything yet */
@@ -577,6 +585,12 @@ gx_default_get_params(gx_device * dev, gs_param_list * plist)
             param_string_from_transient_string(postren_profile,
                 dev_profile->postren_profile->name);
         }
+        if (dev_profile->blend_profile == NULL) {
+            param_string_from_string(blend_profile, null_str);
+        } else {
+            param_string_from_transient_string(blend_profile,
+                dev_profile->blend_profile->name);
+        }
         devicegraytok = dev_profile->devicegraytok;
         graydetection = dev_profile->graydetection;
         usefastcolor = dev_profile->usefastcolor;
@@ -609,6 +623,7 @@ gx_default_get_params(gx_device * dev, gs_param_list * plist)
         param_string_from_string(link_profile, null_str);
         param_string_from_string(icc_colorants, null_str);
         param_string_from_string(postren_profile, null_str);
+        param_string_from_string(blend_profile, null_str);
     }
     /* Transmit the values. */
     /* Standard parameters */
@@ -647,6 +662,7 @@ gx_default_get_params(gx_device * dev, gs_param_list * plist)
         (code = param_write_string(plist,"TextICCProfile", &(profile_array[3]))) < 0 ||
         (code = param_write_string(plist,"ProofProfile", &(proof_profile))) < 0 ||
         (code = param_write_string(plist, "PostRenderProfile", &(postren_profile))) < 0 ||
+        (code = param_write_string(plist, "BlendColorProfile", &(blend_profile))) < 0 ||
         (code = param_write_string(plist,"DeviceLinkProfile", &(link_profile))) < 0 ||
         (code = param_write_string(plist,"ICCOutputColors", &(icc_colorants))) < 0 ||
         (code = param_write_int(plist,"RenderIntent", (const int *) (&(profile_intents[0])))) < 0 ||
@@ -663,7 +679,6 @@ gx_default_get_params(gx_device * dev, gs_param_list * plist)
         (code = param_write_int(plist,"TextKPreserve", (const int *) &(blackpreserve[3]))) < 0 ||
         (code = param_write_int_array(plist, "HWSize", &hwsa)) < 0 ||
         (code = param_write_float_array(plist, ".HWMargins", &hwma)) < 0 ||
-        (code = param_write_float_array(plist, ".MarginsHWResolution", &mhwra)) < 0 ||
         (code = param_write_float_array(plist, ".MediaSize", &msa)) < 0 ||
         (code = param_write_string(plist, "Name", &dns)) < 0 ||
         (code = param_write_int(plist, "Colors", &colors)) < 0 ||
@@ -1313,7 +1328,6 @@ gx_default_put_params(gx_device * dev, gs_param_list * plist)
     gs_param_float_array msa;
     gs_param_float_array ma;
     gs_param_float_array hwma;
-    gs_param_float_array mhwra;
     gs_param_string_array scna;
     int nci = dev->NumCopies;
     int ncset = dev->NumCopies_set;
@@ -1503,15 +1517,6 @@ e:	param_signal_error(plist, param_name, ecode);\
     BEGIN_ARRAY_PARAM(param_read_float_array, ".HWMargins", hwma, 4, hwme) {
         break;
     } END_ARRAY_PARAM(hwma, hwme);
-    /* MarginsHWResolution cannot be changed, only checked. */
-    BEGIN_ARRAY_PARAM(param_read_float_array, ".MarginsHWResolution", mhwra, 2, mhwre) {
-        if (mhwra.data[0] != dev->MarginsHWResolution[0] ||
-            mhwra.data[1] != dev->MarginsHWResolution[1]
-        )
-            ecode = gs_note_error(gs_error_rangecheck);
-        else
-            break;
-    } END_ARRAY_PARAM(mhwra, mhwre);
     switch (code = param_read_bool(plist, (param_name = ".IgnoreNumCopies"), &ignc)) {
         default:
             ecode = code;
@@ -1546,30 +1551,60 @@ nce:
     }
     /* Set the ICC output colors first */
     if ((code = param_read_string(plist, "ICCOutputColors", &icc_pro)) != 1) {
-        gx_default_put_icc_colorants(&icc_pro, dev);
+        if ((code = gx_default_put_icc_colorants(&icc_pro, dev)) < 0) {
+            ecode = code;
+            param_signal_error(plist, "ICCOutputColors", ecode);
+        }
     }
     if ((code = param_read_string(plist, "OutputICCProfile", &icc_pro)) != 1) {
-        gx_default_put_icc(&icc_pro, dev, gsDEFAULTPROFILE);
+        if ((code = gx_default_put_icc(&icc_pro, dev, gsDEFAULTPROFILE)) < 0) {
+            ecode = code;
+            param_signal_error(plist, "OutputICCProfile", ecode);
+        }
     }
     /* Note, if a change is made to NUM_DEVICE_PROFILES we need to update
        this with the name of the profile */
     if ((code = param_read_string(plist, "GraphicICCProfile", &icc_pro)) != 1) {
-        gx_default_put_icc(&icc_pro, dev, gsGRAPHICPROFILE);
+        if ((code = gx_default_put_icc(&icc_pro, dev, gsGRAPHICPROFILE)) < 0) {
+            ecode = code;
+            param_signal_error(plist, "GraphicICCProfile", ecode);
+        }
     }
     if ((code = param_read_string(plist, "ImageICCProfile", &icc_pro)) != 1) {
-        gx_default_put_icc(&icc_pro, dev, gsIMAGEPROFILE);
+        if ((code = gx_default_put_icc(&icc_pro, dev, gsIMAGEPROFILE)) < 0) {
+            ecode = code;
+            param_signal_error(plist, "ImageICCProfile", ecode);
+        }
     }
     if ((code = param_read_string(plist, "TextICCProfile", &icc_pro)) != 1) {
-        gx_default_put_icc(&icc_pro, dev, gsTEXTPROFILE);
+        if ((code = gx_default_put_icc(&icc_pro, dev, gsTEXTPROFILE)) < 0) {
+            ecode = code;
+            param_signal_error(plist, "TextICCProfile", ecode);
+        }
     }
     if ((code = param_read_string(plist, "ProofProfile", &icc_pro)) != 1) {
-        gx_default_put_icc(&icc_pro, dev, gsPROOFPROFILE);
+        if ((code = gx_default_put_icc(&icc_pro, dev, gsPROOFPROFILE)) < 0) {
+            ecode = code;
+            param_signal_error(plist, "ProofProfile", ecode);
+        }
     }
     if ((code = param_read_string(plist, "DeviceLinkProfile", &icc_pro)) != 1) {
-        gx_default_put_icc(&icc_pro, dev, gsLINKPROFILE);
+        if ((code = gx_default_put_icc(&icc_pro, dev, gsLINKPROFILE)) < 0) {
+            ecode = code;
+            param_signal_error(plist, "DeviceLinkProfile", ecode);
+        }
     }
     if ((code = param_read_string(plist, "PostRenderProfile", &icc_pro)) != 1) {
-        gx_default_put_icc(&icc_pro, dev, gsPRPROFILE);
+        if ((code = gx_default_put_icc(&icc_pro, dev, gsPRPROFILE)) < 0) {
+            ecode = code;
+            param_signal_error(plist, "PostRenderProfile", ecode);
+        }
+    }
+    if ((code = param_read_string(plist, "BlendColorProfile", &icc_pro)) != 1) {
+        if ((code = gx_default_put_icc(&icc_pro, dev, gsBLENDPROFILE)) < 0) {
+            ecode = code;
+            param_signal_error(plist, "BlendColorProfile", ecode);
+        }
     }
     if ((code = param_read_int(plist, (param_name = "RenderIntent"),
                                                     &(rend_intent[0]))) < 0) {
