@@ -18,6 +18,8 @@
 
 #include "memory_.h"
 #include "gx.h"
+#include "gsstruct.h"
+#include "gxobj.h"
 #include "gsstype.h"
 #include "gserrors.h"
 #include "gsmchunk.h"
@@ -81,6 +83,8 @@ static const gs_memory_procs_t chunk_procs =
 
 typedef struct chunk_obj_node_s {
     struct chunk_obj_node_s *next;
+    struct chunk_obj_node_s *prev;
+    struct chunk_mem_node_s *chunk;
     gs_memory_type_ptr_t type;
     uint size;			/* objlist: client size */
 #ifdef DEBUG
@@ -88,6 +92,11 @@ typedef struct chunk_obj_node_s {
 #endif
 } chunk_obj_node_t;
 
+typedef struct chunk_freelist_node_s {
+    struct chunk_freelist_node_s *next;
+    uint size;			/* size of entire freelist block */
+} chunk_freelist_node_t;
+
 /*
  * Note: All objects within a chunk are 'aligned' since we round_up_to_align
  * the free list pointer when removing part of a free area.
@@ -95,11 +104,14 @@ typedef struct chunk_obj_node_s {
 typedef struct chunk_mem_node_s {
     uint size;
     uint largest_free;			/* quick check when allocating */
+                                        /* NB: largest_free INCLUDES the size of the obj_node */
     bool is_multiple_object_chunk;	/* tells us which list this chunk is on */
     struct chunk_mem_node_s *next;
+    struct chunk_mem_node_s *prev;
     chunk_obj_node_t *objlist;	/* head of objects in this chunk (no order) */
-    chunk_obj_node_t *freelist;		/* free list (ordered) */
-    /* chunk data follows immediately */
+    chunk_freelist_node_t *freelist;    /* free list (ordered) */
+                                        /* size INCLUDES freelist_node structure */
+    /* chunk data follows on the next obj_align_mod aligned boundary */
 } chunk_mem_node_t;
 
 typedef struct gs_memory_chunk_s {
@@ -108,13 +120,15 @@ typedef struct gs_memory_chunk_s {
     chunk_mem_node_t *head_mo_chunk;	/* head of multiple object chunks */
     chunk_mem_node_t *head_so_chunk;	/* head of single object chunks */
     unsigned long used;
+    unsigned long max_used;
 #ifdef DEBUG
     unsigned long sequence_counter;
-    unsigned long max_used;
     int	     in_use;		/* 0 for idle, 1 in alloc, -1 in free */
 #endif
 } gs_memory_chunk_t;
 
+#define SIZEOF_ROUND_ALIGN(a) ROUND_UP(sizeof(a), obj_align_mod)
+
 /* ---------- Public constructors/destructors ---------- */
 
 /* Initialize a gs_memory_chunk_t */
@@ -141,9 +155,9 @@ gs_memory_chunk_wrap( gs_memory_t **wrapped,	/* chunk allocator init */
     cmem->head_mo_chunk = NULL;
     cmem->head_so_chunk = NULL;
     cmem->used = 0;
+    cmem->max_used = 0;
 #ifdef DEBUG
     cmem->sequence_counter = 0;
-    cmem->max_used = 0;
     cmem->in_use = 0;		/* idle */
 #endif
 
@@ -161,6 +175,22 @@ gs_memory_chunk_release(gs_memory_t *mem)
                        "gs_memory_chunk_release");
 }
 
+/* Release chunk memory manager, and return the target */
+gs_memory_t * /* Always succeeds */
+gs_memory_chunk_unwrap(gs_memory_t *mem)
+{
+    gs_memory_t *tmem;
+
+    if (mem->procs.status != chunk_status) {
+        tmem = mem;
+    }
+    else {
+        tmem = ((gs_memory_chunk_t *)mem)->target;
+        gs_memory_chunk_release(mem);
+    }
+    return tmem;
+}
+
 /* ---------- Accessors ------------- */
 
 /* Retrieve this allocator's target */
@@ -181,10 +211,10 @@ gs_memory_chunk_dump_memory(const gs_memory_t *mem)
     chunk_mem_node_t *next;
     int i;
 
-    dprintf2("chunk_dump_memory: current used=%d, max_used=%d\n", cmem->used, cmem->max_used);
+    dmprintf2(mem, "chunk_dump_memory: current used=%lu, max_used=%lu\n", cmem->used, cmem->max_used);
     if (cmem->in_use != 0)
-        dprintf1("*** this memory allocator is not idle, used for: %s\n",
-                cmem->in_use < 0 ? "free" : "alloc");
+        dmprintf1(mem, "*** this memory allocator is not idle, used for: %s\n",
+                  cmem->in_use < 0 ? "free" : "alloc");
     for (i=0; i<2; i++) {
         current = head;
         while ( current != NULL ) {
@@ -192,8 +222,8 @@ gs_memory_chunk_dump_memory(const gs_memory_t *mem)
                 chunk_obj_node_t *obj;
 
                 for (obj= current->objlist; obj != NULL; obj=obj->next)
-                    dprintf4("chunk_mem leak, obj=0x%lx, size=%d, type=%s, sequence#=%ld\n",
-                            (ulong)obj, obj->size, obj->type->sname, obj->sequence);
+                    dmprintf4(mem, "chunk_mem leak, obj=0x%lx, size=%d, type=%s, sequence#=%ld\n",
+                              (ulong)obj, obj->size, obj->type->sname, obj->sequence);
             }
             next = current->next;
             current = next;
@@ -222,8 +252,8 @@ chunk_mem_node_free_all_remaining(gs_memory_chunk_t *cmem)
 
 #ifdef DEBUG
     if (cmem->in_use != 0)
-        dprintf1("*** chunk_mem_node_free_all_remaining: this memory allocator is not idle, used for: %s\n",
-                cmem->in_use < 0 ? "free" : "alloc");
+        dmprintf1(target, "*** chunk_mem_node_free_all_remaining: this memory allocator is not idle, used for: %s\n",
+                  cmem->in_use < 0 ? "free" : "alloc");
 #endif
     for (i=0; i<2; i++) {
         current = head;
@@ -246,8 +276,8 @@ chunk_free_all(gs_memory_t * mem, uint free_mask, client_name_t cname)
 
 #ifdef DEBUG
     if (cmem->in_use != 0)
-        dprintf1("*** chunk_free_all: this memory allocator is not idle, used for: %s\n",
-                cmem->in_use < 0 ? "free" : "alloc");
+        dmprintf1(target, "*** chunk_free_all: this memory allocator is not idle, used for: %s\n",
+                  cmem->in_use < 0 ? "free" : "alloc");
 #endif
     /* Only free the structures and the allocator itself. */
     if (mem->stable_memory) {
@@ -272,9 +302,9 @@ extern const gs_memory_struct_type_t st_bytes;
 inline static uint
 round_up_to_align(uint size)
 {
-    uint num_node_headers = (size + sizeof(chunk_obj_node_t) - 1) / sizeof(chunk_obj_node_t);
+    uint num_node_headers = (size + SIZEOF_ROUND_ALIGN(chunk_obj_node_t) - 1) / SIZEOF_ROUND_ALIGN(chunk_obj_node_t);
 
-    return num_node_headers * sizeof(chunk_obj_node_t);
+    return num_node_headers * SIZEOF_ROUND_ALIGN(chunk_obj_node_t);
 }
 
 #ifdef MEMENTO
@@ -291,18 +321,14 @@ round_up_to_align(uint size)
 /* return -1 on error, 0 on success */
 static int
 chunk_mem_node_add(gs_memory_chunk_t *cmem, uint size_needed, bool is_multiple_object_chunk,
-                        chunk_mem_node_t **newchunk
-#ifdef MEMENTO
-                        , client_name_t cname
-#endif
-                  )
+                        chunk_mem_node_t **newchunk, client_name_t cname)
 {
     chunk_mem_node_t *node;
     gs_memory_t *target = cmem->target;
     /* Allocate enough for the chunk header, and the size_needed */
     /* The size needed already includes the object header from caller */
     /* and is already rounded up to the obj_node_t sized elements */
-    uint chunk_size = size_needed + sizeof(chunk_mem_node_t);
+    uint chunk_size = size_needed + SIZEOF_ROUND_ALIGN(chunk_mem_node_t);
 
     /* caller tells us whether or not to use a single object chunk */
     if (is_multiple_object_chunk && (chunk_size < MULTIPLE_OBJ_CHUNK_SIZE)) {
@@ -312,37 +338,30 @@ chunk_mem_node_add(gs_memory_chunk_t *cmem, uint size_needed, bool is_multiple_o
         is_multiple_object_chunk = false;
 
     *newchunk = NULL;
-#ifdef MEMENTO
-#define LOCAL_CNAME cname
-#else
-#define LOCAL_CNAME "chunk_mem_node_add"
-#endif
-    node = (chunk_mem_node_t *)gs_alloc_bytes_immovable(target, chunk_size,
-                                                        LOCAL_CNAME);
-#undef LOCAL_CNAME
+    node = (chunk_mem_node_t *)gs_alloc_bytes_immovable(target, chunk_size, cname);
     if (node == NULL)
         return -1;
     cmem->used += chunk_size;
-#ifdef DEBUG
     if (cmem->used > cmem->max_used)
         cmem->max_used = cmem->used;
-#endif
     node->size = chunk_size;	/* how much we allocated */
-    node->largest_free = chunk_size - sizeof(chunk_mem_node_t);
+    node->largest_free = chunk_size - SIZEOF_ROUND_ALIGN(chunk_mem_node_t);
     node->is_multiple_object_chunk = is_multiple_object_chunk;
     node->objlist = NULL;
-    node->freelist = (chunk_obj_node_t *)((byte *)(node) + sizeof(chunk_mem_node_t));
+    node->freelist = (chunk_freelist_node_t *)((byte *)(node) + SIZEOF_ROUND_ALIGN(chunk_mem_node_t));
     node->freelist->next = NULL;
     node->freelist->size = node->largest_free;
 
     /* Put the node at the head of the list (so=single object, mo=multiple object) */
     /* only multiple objects will be have any room in them */
+    node->prev = NULL;			/* head of list always has prev == NULL */
     if (is_multiple_object_chunk) {
         if (cmem->head_mo_chunk == NULL) {
             cmem->head_mo_chunk = node;
             node->next = NULL;
         } else {
             node->next = cmem->head_mo_chunk;
+            cmem->head_mo_chunk->prev = node;
             cmem->head_mo_chunk = node;
         }
     } else {
@@ -351,6 +370,7 @@ chunk_mem_node_add(gs_memory_chunk_t *cmem, uint size_needed, bool is_multiple_o
             node->next = NULL;
         } else {
             node->next = cmem->head_so_chunk;
+            cmem->head_so_chunk->prev = node;
             cmem->head_so_chunk = node;
         }
     }
@@ -373,29 +393,24 @@ chunk_mem_node_remove(gs_memory_chunk_t *cmem, chunk_mem_node_t *addr)
 
     /* check the head first */
     if (head == NULL) {
-        dprintf("FAIL - no nodes to be removed\n" );
+        dmprintf(target, "FAIL - no nodes to be removed\n" );
         return -1;
     }
     if (head == addr) {
         *p_head = head->next;
+        if (head->next != NULL)
+            head->next->prev = NULL;
         gs_free_object(target, head, "chunk_mem_node_remove");
     } else {
         chunk_mem_node_t *current;
-        bool found = false;
 
-        /* scan the list, stopping in front of element */
-        for (current = head; current != NULL; current = current->next) {
-            if ( current->next && (current->next == addr) ) {
-                current->next = current->next->next;	/* de-link it */
-                gs_free_object(target, addr, "chunk_mem_node_remove");
-                found = true;
-                break;
-            }
-        }
-        if ( !found ) {
-            dprintf1("FAIL freeing wild pointer freed address 0x%lx not found\n", (ulong)addr );
-            return -1;
+        current = addr->prev;
+        current->next = addr->next;
+        if (addr->next != NULL) {
+            addr->next->prev = current;
         }
+        gs_free_object(target, addr, "chunk_mem_node_remove");
+
     }
     return 0;
 }
@@ -408,18 +423,22 @@ chunk_obj_alloc(gs_memory_t *mem, uint size, gs_memory_type_ptr_t type, client_n
     chunk_mem_node_t *head = cmem->head_mo_chunk;	/* we only scan chunks with space in them */
     uint newsize, free_size;
     chunk_obj_node_t *newobj = NULL;
-    chunk_obj_node_t *free_obj, *prev_free, *new_free;
+    chunk_freelist_node_t *free_obj, *prev_free, *new_free;
     chunk_mem_node_t *current = NULL;
     bool rescan_free_list = false;
     bool is_multiple_object_size;
 
 #ifdef DEBUG
     if (cmem->in_use != 0)
-        dprintf1("*** chunk_obj_alloc: this memory allocator is not idle, used for: %s\n",
+        dmprintf1(mem, "*** chunk_obj_alloc: this memory allocator is not idle, used for: %s\n",
                 cmem->in_use < 0 ? "free" : "alloc");
     cmem->in_use = 1;	/* alloc */
 #endif
-    newsize = round_up_to_align(size + sizeof(chunk_obj_node_t));	/* space we will need */
+    newsize = round_up_to_align(size + SIZEOF_ROUND_ALIGN(chunk_obj_node_t));	/* space we will need */
+    /* Protect against overflow */
+    if (newsize < size) {
+        return NULL;
+    }
     is_multiple_object_size = ! IS_SINGLE_OBJ_SIZE(newsize);
 
     if ( is_multiple_object_size ) {
@@ -431,14 +450,12 @@ chunk_obj_alloc(gs_memory_t *mem, uint size, gs_memory_type_ptr_t type, client_n
     }
     if (current == NULL) {
         /* No chunks with enough space or size makes this a single object, allocate one */
-        if (chunk_mem_node_add(cmem, newsize, is_multiple_object_size, &current
-#ifdef MEMENTO
-                               , cname
-#endif
-                              ) < 0) {
+        /* If this object is in its own chunk, use the caller's cname when allocating the chunk */
+        if (chunk_mem_node_add(cmem, newsize, is_multiple_object_size, &current,
+                               is_multiple_object_size ? "chunk_mem_node_add" : cname) < 0) {
 #ifdef DEBUG
         if (gs_debug_c('a'))
-            dlprintf1("[a+]chunk_obj_alloc(chunk_mem_node_add)(%u) Failed.\n", size);
+            dmlprintf1(mem, "[a+]chunk_obj_alloc(chunk_mem_node_add)(%u) Failed.\n", size);
             cmem->in_use = 0;	/* idle */
 #endif
             return NULL;
@@ -454,7 +471,7 @@ chunk_obj_alloc(gs_memory_t *mem, uint size, gs_memory_type_ptr_t type, client_n
     }
 
     if (free_obj == NULL) {
-        dprintf2("largest_free value = %d is too large, cannot find room for size = %d\n",
+        dmprintf2(mem, "largest_free value = %d is too large, cannot find room for size = %d\n",
             current->largest_free, newsize);
 #ifdef DEBUG
         cmem->in_use = 0;	/* idle */
@@ -467,9 +484,10 @@ chunk_obj_alloc(gs_memory_t *mem, uint size, gs_memory_type_ptr_t type, client_n
 
     /* Make an object in the free_obj we found above, reducing it's size */
     /* and adjusting the free list preserving alignment	*/
-    newobj = free_obj;
+    newobj = (chunk_obj_node_t *)free_obj;
     free_size = free_obj->size - newsize;	/* amount remaining */
-    new_free = (chunk_obj_node_t *)((byte *)(free_obj) + newsize);	/* start of remaining free area */
+    new_free = (chunk_freelist_node_t *)((byte *)(free_obj) + newsize);	/* start of remaining free area */
+
     if (free_size >= sizeof(chunk_obj_node_t)) {
         if (prev_free != NULL)
             prev_free->next = new_free;
@@ -477,6 +495,11 @@ chunk_obj_alloc(gs_memory_t *mem, uint size, gs_memory_type_ptr_t type, client_n
             current->freelist = new_free;
         new_free->next = free_obj->next;
         new_free->size = free_size;
+        if (rescan_free_list && current->freelist->next == NULL) {
+            /* Object was allocated from the single freelist entry, so adjust largest */
+            current->largest_free = free_size;
+            rescan_free_list = false;
+        }
     } else {
        /* Not enough space remaining, just skip around it */
         if (prev_free != NULL)
@@ -485,13 +508,19 @@ chunk_obj_alloc(gs_memory_t *mem, uint size, gs_memory_type_ptr_t type, client_n
             current->freelist = free_obj->next;
     }
 
+
+
 #ifdef DEBUG
-    memset((byte *)(newobj) + sizeof(chunk_obj_node_t), 0xa1, newsize - sizeof(chunk_obj_node_t));
-    memset((byte *)(newobj) + sizeof(chunk_obj_node_t), 0xac, size);
+    memset((byte *)(newobj) + SIZEOF_ROUND_ALIGN(chunk_obj_node_t), 0xa1, newsize - SIZEOF_ROUND_ALIGN(chunk_obj_node_t));
+    memset((byte *)(newobj) + SIZEOF_ROUND_ALIGN(chunk_obj_node_t), 0xac, size);
     newobj->sequence = cmem->sequence_counter++;
 #endif
 
     newobj->next = current->objlist;	/* link to start of list */
+    newobj->chunk = current;
+    newobj->prev = NULL;
+    if (current->objlist)
+        current->objlist->prev = newobj;
     current->objlist = newobj;
     newobj->size = size;		/* client requested size */
     newobj->type = type;		/* and client desired type */
@@ -507,11 +536,11 @@ chunk_obj_alloc(gs_memory_t *mem, uint size, gs_memory_type_ptr_t type, client_n
     /* return the client area of the object we allocated */
 #ifdef DEBUG
     if (gs_debug_c('A'))
-        dlprintf3("[a+]chunk_obj_alloc (%s)(%u) = 0x%lx: OK.\n",
-                  client_name_string(cname), size, (ulong) newobj);
+        dmlprintf3(mem, "[a+]chunk_obj_alloc (%s)(%u) = 0x%lx: OK.\n",
+                   client_name_string(cname), size, (ulong) newobj);
     cmem->in_use = 0; 	/* idle */
 #endif
-    return (byte *)(newobj) + sizeof(chunk_obj_node_t);
+    return (byte *)(newobj) + SIZEOF_ROUND_ALIGN(chunk_obj_node_t);
 }
 
 static byte *
@@ -572,16 +601,14 @@ static void *
 chunk_resize_object(gs_memory_t * mem, void *ptr, uint new_num_elements, client_name_t cname)
 {
     /* This isn't particularly efficient, but it is rarely used */
-    chunk_obj_node_t *obj = ((chunk_obj_node_t *)ptr) - 1;
+    chunk_obj_node_t *obj = (chunk_obj_node_t *)(((byte *)ptr) - SIZEOF_ROUND_ALIGN(chunk_obj_node_t));
     ulong new_size = (obj->type->ssize * new_num_elements);
     ulong old_size = obj->size;
     /* get the type from the old object */
     gs_memory_type_ptr_t type = obj->type;
     void *new_ptr;
-#ifdef DEBUG
     gs_memory_chunk_t *cmem = (gs_memory_chunk_t *)mem;
     ulong save_max_used = cmem->max_used;
-#endif
 
     if (new_size == old_size)
         return ptr;
@@ -589,11 +616,9 @@ chunk_resize_object(gs_memory_t * mem, void *ptr, uint new_num_elements, client_
         return 0;
     memcpy(new_ptr, ptr, min(old_size, new_size));
     chunk_free_object(mem, ptr, cname);
-#ifdef DEBUG
     cmem->max_used = save_max_used;
     if (cmem->used > cmem->max_used)
         cmem->max_used = cmem->used;
-#endif
     return new_ptr;
 }
 
@@ -606,63 +631,30 @@ chunk_free_object(gs_memory_t * mem, void *ptr, client_name_t cname)
         return;
     {
         /* back up to obj header */
-        chunk_obj_node_t *obj = ((chunk_obj_node_t *)ptr) - 1;
+        int obj_node_size = SIZEOF_ROUND_ALIGN(chunk_obj_node_t);
+        chunk_obj_node_t *obj = (chunk_obj_node_t *)(((byte *)ptr) - obj_node_size);
         struct_proc_finalize((*finalize)) = obj->type->finalize;
         chunk_mem_node_t *current;
-        chunk_obj_node_t *free_obj, *prev_free;
-        chunk_obj_node_t *scan_obj, *prev_obj;
+        chunk_freelist_node_t *free_obj, *prev_free, *new_free;
+        chunk_obj_node_t *prev_obj;
         /* space we will free */
-        uint freed_size = round_up_to_align(obj->size + sizeof(chunk_obj_node_t));
+        uint freed_size = round_up_to_align(obj->size + obj_node_size);
 
         if ( finalize != NULL )
             finalize(mem, ptr);
 #ifdef DEBUG
         if (cmem->in_use != 0)
-            dprintf1("*** chunk_free_object: this memory allocator is not idle, used for: %s\n",
-                    cmem->in_use < 0 ? "free" : "alloc");
+            dmprintf1(cmem->target,
+                      "*** chunk_free_object: this memory allocator is not idle, used for: %s\n",
+                      cmem->in_use < 0 ? "free" : "alloc");
         cmem->in_use = -1;	/* free */
 #endif
         /* finalize may change the head_**_chunk doing free of stuff */
-        current = IS_SINGLE_OBJ_SIZE(freed_size) ?
-                                        cmem->head_so_chunk : cmem->head_mo_chunk;
-        /* Find the chunk containing this object */
-        for ( ; current != NULL; current = current->next) {
-            if (((byte *)obj > (byte *)current) && ((byte *)obj < (byte *)(current) + current->size))
-                break;
-        }
-        if (current == NULL) {
-            /* We _may_have searched the wrong list -- if so find out. */
-            current = cmem->head_so_chunk;
-            /* Find the chunk containing this object */
-            for ( ; current != NULL; current = current->next) {
-                if (((byte *)obj > (byte *)current) && ((byte *)obj < (byte *)(current) + current->size)) {
-                    dprintf1("chunk_free_obj: OOPS! found it on the single_object list, size=%d\n",
-                                obj->size);
-                    break;
-                }
-            }
-            if (current == NULL) {
-                current = cmem->head_mo_chunk;
-                /* Find the chunk containing this object */
-                for ( ; current != NULL; current = current->next) {
-                    if (((byte *)obj > (byte *)current) && ((byte *)obj < (byte *)(current) + current->size)) {
-                        dprintf1("chunk_free_obj: OOPS! found it on the multiple_object list, size=%d\n",
-                                obj->size);
-                        break;
-                    }
-                }
-            }
-            if (current == NULL) {
-                /* Object not found in any chunk */
-                dprintf2("chunk_free_obj failed, object 0x%lx not in any chunk, size=%d\n", ((ulong)obj), obj->size);
-#ifdef DEBUG
-                cmem->in_use = 0; 	/* idle */
-#endif
-                return;
-            }
-        }
+        current = obj->chunk;
+
         /* For large objects, they were given their own chunk -- just remove the node */
-        if (IS_SINGLE_OBJ_SIZE(freed_size)) {
+        /* For multiple object chunks, we can remove the node if the objlist will become empty */
+        if (current->is_multiple_object_chunk == 0 || current->objlist->next == NULL) {
             chunk_mem_node_remove(cmem, current);
 #ifdef DEBUG
             cmem->in_use = 0; 	/* idle */
@@ -670,60 +662,50 @@ chunk_free_object(gs_memory_t * mem, void *ptr, client_name_t cname)
             return;
         }
 
-        /* Scan obj list to find this element */
-        prev_obj = NULL;	/* object is head, linked to mem node */
-        for (scan_obj = current->objlist; scan_obj != NULL; scan_obj = scan_obj->next) {
-            if (scan_obj == obj)
-                break;
-            prev_obj = scan_obj;
-        }
-        if (scan_obj == NULL) {
-            /* Object not found in expected chunk */
-            dprintf3("chunk_free_obj failed, object 0x%lx not in chunk at 0x%lx, size = %d\n",
-                            ((ulong)obj), ((ulong)current), current->size);
-#ifdef DEBUG
-            cmem->in_use = 0; 	/* idle */
-#endif
-            return;
-        }
+        prev_obj = obj->prev;
         /* link around the object being freed */
-        if (prev_obj == NULL)
+        if (prev_obj == NULL) {
             current->objlist = obj->next;
-        else
+            if (obj->next)
+                obj->next->prev = NULL;
+        } else {
             prev_obj->next = obj->next;
+            if (obj->next)
+                obj->next->prev = prev_obj;
+        }
 
-        if_debug3('A', "[a-]chunk_free_object(%s) 0x%lx(%u)\n",
+        if_debug3m('A', cmem->target, "[a-]chunk_free_object(%s) 0x%lx(%u)\n",
                   client_name_string(cname), (ulong) ptr, obj->size);
 
         /* Add this object's space (including the header) to the free list */
-
         /* Scan free list to find where this element goes */
-        obj->size = freed_size;	    /* adjust size to include chunk_obj_node and pad */
-
         prev_free = NULL;
+        new_free = (chunk_freelist_node_t *)obj;
+        new_free->size = freed_size;        /* set the size of this free block */
         for (free_obj = current->freelist; free_obj != NULL; free_obj = free_obj->next) {
-            if (obj < free_obj)
+            if (new_free < free_obj)
                 break;
             prev_free = free_obj;
         }
         if (prev_free == NULL) {
             /* this object is before any other free objects */
-            obj->next = current->freelist;
-            current->freelist = obj;
+            new_free->next = current->freelist;
+            current->freelist = new_free;
         } else {
-            obj->next = free_obj;
-            prev_free->next = obj;
+            new_free->next = free_obj;
+            prev_free->next = new_free;
         }
         /* If the end of this object is adjacent to the next free space,
          * merge the two. Next we'll merge with predecessor (prev_free)
          */
         if (free_obj != NULL) {
-            byte *after_obj = (byte*)(obj) + freed_size;
+            byte *after_obj = (byte*)(new_free) + freed_size;
 
-            if (free_obj <= (chunk_obj_node_t *)after_obj) {
+            if (free_obj <= (chunk_freelist_node_t *)after_obj) {
                 /* Object is adjacent to following free space block -- merge it */
-                obj->next = free_obj->next;	/* link around the one being absorbed */
-                obj->size = (byte *)(free_obj) - (byte *)(obj) + free_obj->size;
+                new_free->next = free_obj->next;	/* link around the one being absorbed */
+                /* as noted elsewhere, freelist node size is entire block size */
+                new_free->size = (byte *)(free_obj) - (byte *)(new_free) + free_obj->size;
             }
         }
         /* the prev_free object precedes this object that is now free,
@@ -732,28 +714,22 @@ chunk_free_object(gs_memory_t * mem, void *ptr, client_name_t cname)
         if (prev_free != NULL) {
             byte *after_free = (byte*)(prev_free) + prev_free->size;
 
-            if (obj <= (chunk_obj_node_t *)after_free) {
+            if (new_free <= (chunk_freelist_node_t *)after_free) {
                 /* Object is adjacent to prior free space block -- merge it */
                 /* NB: this is the common case with LIFO alloc-free patterns */
                 /* (LIFO: Last-allocated, first freed) */
-                prev_free->size = (byte *)(obj) - (byte *)(prev_free) + obj->size;
-                prev_free->next = obj->next;		/* link around 'obj' area */
-                obj = prev_free;
+                prev_free->size = (byte *)(new_free) - (byte *)(prev_free) + new_free->size;
+                prev_free->next = new_free->next;		/* link around 'new_free' area */
+                new_free = prev_free;
             }
         }
 #ifdef DEBUG
-memset((byte *)(obj) + sizeof(chunk_obj_node_t), 0xf1, obj->size - sizeof(chunk_obj_node_t));
+memset((byte *)(new_free) + SIZEOF_ROUND_ALIGN(chunk_freelist_node_t), 0xf1,
+       new_free->size - SIZEOF_ROUND_ALIGN(chunk_freelist_node_t));
 #endif
-        if (current->largest_free < obj->size)
-            current->largest_free = obj->size;
+        if (current->largest_free < new_free->size)
+            current->largest_free = new_free->size;
 
-        /* If this chunk is now totally empty, free it */
-        if (current->objlist == NULL) {
-            if (current->size != current->freelist->size + sizeof(chunk_mem_node_t))
-                dprintf2("chunk freelist size not correct, is: %d, should be: %d\n",
-                    round_up_to_align(current->freelist->size + sizeof(chunk_mem_node_t)), current->size);
-            chunk_mem_node_remove(cmem, current);
-        }
 #ifdef DEBUG
         cmem->in_use = 0; 	/* idle */
 #endif
@@ -794,7 +770,7 @@ chunk_status(gs_memory_t * mem, gs_memory_status_t * pstat)
 {
     gs_memory_chunk_t *cmem = (gs_memory_chunk_t *)mem;
     chunk_mem_node_t *current = cmem->head_mo_chunk;	/* we only scan chunks with space in them */
-    chunk_obj_node_t *free_obj;		/* free list object node */
+    chunk_freelist_node_t *free_obj;		/* free list object node */
     int tot_free = 0;
 
     pstat->allocated = cmem->used;
@@ -804,6 +780,7 @@ chunk_status(gs_memory_t * mem, gs_memory_status_t * pstat)
             tot_free += free_obj->size;
     }
     pstat->used = cmem->used - tot_free;
+    pstat->max_used = cmem->max_used;
 
     pstat->is_thread_safe = false;	/* this allocator does not have an internal mutex */
 }
@@ -828,7 +805,7 @@ chunk_consolidate_free(gs_memory_t *mem)
 static uint
 chunk_object_size(gs_memory_t * mem, const void *ptr)
 {
-    chunk_obj_node_t *obj = ((chunk_obj_node_t *)ptr) - 1;
+    chunk_obj_node_t *obj = (chunk_obj_node_t *)(((byte *)ptr) - SIZEOF_ROUND_ALIGN(chunk_obj_node_t));
 
     return obj->size;
 }
@@ -836,7 +813,7 @@ chunk_object_size(gs_memory_t * mem, const void *ptr)
 static gs_memory_type_ptr_t
 chunk_object_type(const gs_memory_t * mem, const void *ptr)
 {
-    chunk_obj_node_t *obj = ((chunk_obj_node_t *)ptr) - 1;
+    chunk_obj_node_t *obj = (chunk_obj_node_t *)(((byte *)ptr) - SIZEOF_ROUND_ALIGN(chunk_obj_node_t));
     return obj->type;
 }
 
@@ -856,7 +833,7 @@ chunk_unregister_root(gs_memory_t * mem, gs_gc_root_t * rp, client_name_t cname)
 
 #define A(obj, size) \
     if ((obj = gs_alloc_bytes(cmem, size, "chunk_alloc_unit_test")) == NULL) { \
-        dprintf("chunk alloc failed\n"); \
+        dmprintf(cmem, "chunk alloc failed\n"); \
         return_error(gs_error_VMerror); \
     }
 
@@ -871,7 +848,7 @@ chunk_allocator_unit_test(gs_memory_t *mem)
     byte *obj1, *obj2, *obj3, *obj4, *obj5, *obj6, *obj7;
 
     if ((code = gs_memory_chunk_wrap(&cmem, mem )) < 0) {
-        dprintf1("chunk_wrap returned error code: %d\n", code);
+        dmprintf1(cmem, "chunk_wrap returned error code: %d\n", code);
         return code;
     }
 
