@@ -598,14 +598,18 @@ gs_pattern1_get_pattern(const gs_pattern_instance_t *pinst)
 void *
 gx_pattern1_get_transptr(const gx_device_color *pdevc)
 {
-    return pdevc->colors.pattern.p_tile->ttrans;
+    if (pdevc->colors.pattern.p_tile != NULL)
+        return pdevc->colors.pattern.p_tile->ttrans;
+    else
+        return NULL;
 }
 
 /* Check for if the clist in the pattern has transparency */
 int
 gx_pattern1_clist_has_trans(const gx_device_color *pdevc)
 {
-    if (pdevc->colors.pattern.p_tile->cdev != NULL) {
+    if (pdevc->colors.pattern.p_tile != NULL &&
+        pdevc->colors.pattern.p_tile->cdev != NULL) {
         return pdevc->colors.pattern.p_tile->cdev->common.page_uses_transparency;
     } else {
         return 0;
@@ -820,6 +824,8 @@ image_PaintProc(const gs_client_color * pcolor, gs_gstate * pgs)
 
     if (ppmap->pcspace == 0) {
         pcspace = gs_cspace_new_DeviceGray(pgs->memory);
+        if (pcspace == NULL)
+            return_error(gs_error_VMerror);
     } else
         pcspace = ppmap->pcspace;
     code = gs_gsave(pgs);
@@ -939,6 +945,8 @@ int pixmap_high_level_pattern(gs_gstate * pgs)
         code = image_PaintProc(&pdc->ccolor, pgs);
     else {
         pcs = gs_cspace_new_DeviceGray(pgs->memory);
+        if (pcs == NULL)
+            return_error(gs_error_VMerror);
         gs_setcolorspace(pgs, pcs);
         code = mask_PaintProc(&pdc->ccolor, pgs);
     }
@@ -1603,7 +1611,7 @@ enum {
 };
 
 static int
-gx_dc_pattern_write_raster(gx_color_tile *ptile, int64_t offset, byte *data, 
+gx_dc_pattern_write_raster(gx_color_tile *ptile, int64_t offset, byte *data,
                            uint *psize, const gx_device *dev)
 {
     int size_b, size_c;
@@ -1611,7 +1619,7 @@ gx_dc_pattern_write_raster(gx_color_tile *ptile, int64_t offset, byte *data,
     int left = *psize;
     int64_t offset1 = offset;
 
-    size_b = sizeof(gx_strip_bitmap) + 
+    size_b = sizeof(gx_strip_bitmap) +
          ptile->tbits.size.y * ptile->tbits.raster * ptile->tbits.num_planes;
     size_c = ptile->tmask.data ? sizeof(gx_strip_bitmap) + ptile->tmask.size.y * ptile->tmask.raster : 0;
     if (data == NULL) {
