@@ -33,6 +33,8 @@ typedef enum {
     PDF14_ABORT_DEVICE,
     PDF14_BEGIN_TRANS_GROUP,
     PDF14_END_TRANS_GROUP,
+    PDF14_BEGIN_TRANS_TEXT_GROUP,
+    PDF14_END_TRANS_TEXT_GROUP,
     PDF14_BEGIN_TRANS_MASK,
     PDF14_END_TRANS_MASK,
     PDF14_SET_BLEND_PARAMS,
@@ -49,6 +51,8 @@ typedef enum {
     "PDF14_ABORT_DEVICE     ",\
     "PDF14_BEGIN_TRANS_GROUP",\
     "PDF14_END_TRANS_GROUP  ",\
+    "PDF14_BEGIN_TRANS_TEXT_GROUP  ",\
+    "PDF14_END_TRANS_TEXT_GROUP  ",\
     "PDF14_BEGIN_TRANS_MASK ",\
     "PDF14_END_TRANS_MASK   ",\
     "PDF14_SET_BLEND_PARAMS ",\
@@ -65,7 +69,16 @@ typedef enum {
 #define PDF14_SET_OPACITY_ALPHA (1 << 3)
 #define PDF14_SET_OVERPRINT		(1 << 4)
 #define PDF14_SET_OVERPRINT_MODE (1 << 5)
-#define PDF14_SET_OVERPRINT_BLEND (1 << 6)
+
+/* Used for keeping track of the text group madness, since we have the pdf14
+   device needs to know if we are int an BT/ET group vs. a FreeText Annotation
+   as well as if we have already pushed a special knockout non-isolated group
+   for doing text knockout */
+typedef enum {
+    PDF14_TEXTGROUP_NO_BT,  /* We are not in a BT/ET.  Avoids Annotation Texts */
+    PDF14_TEXTGROUP_BT_NOT_PUSHED, /* We are in a BT/ET but no group pushed */
+    PDF14_TEXTGROUP_BT_PUSHED   /* We are in a BT/ET section and group was pushed */
+} pdf14_text_group_state;
 
 #ifndef gs_function_DEFINED
 typedef struct gs_function_s gs_function_t;
@@ -107,6 +120,7 @@ struct gs_pdf14trans_params_s {
     /* Individual transparency parameters */
     gs_blend_mode_t blend_mode;
     bool text_knockout;
+    int text_group;
     gs_transparency_source_t opacity;
     gs_transparency_source_t shape;
     bool mask_is_image;
@@ -114,7 +128,6 @@ struct gs_pdf14trans_params_s {
     bool replacing;
     bool overprint;
     bool overprint_mode;
-    bool blendspot;
     bool idle; /* For clist reader.*/
     uint mask_id; /* For clist reader.*/
     int group_color_numcomps;
@@ -171,6 +184,9 @@ int gs_begin_transparency_group(gs_gstate * pgs,
 
 int gs_end_transparency_group(gs_gstate *pgs);
 
+int gs_end_transparency_text_group(gs_gstate *pgs);
+int gs_begin_transparency_text_group(gs_gstate *pgs);
+
 void gs_trans_mask_params_init(gs_transparency_mask_params_t *ptmp,
                                gs_transparency_mask_subtype_t subtype);
 
@@ -231,13 +247,13 @@ int gs_is_pdf14trans_compositor(const gs_composite_t * pct);
  * since it is a hack, we may exceed our desired buffer space while
  * processing the file.
  */
-#define NUM_PDF14_BUFFERS 3     /* totally a random guess */
+#define NUM_PDF14_BUFFERS 4     /* totally a random guess */
 #define NUM_ALPHA_CHANNELS 1    /* common, but doesn't include possible tag, shape or group alpha */
 #define NUM_COLOR_CHANNELS 4    /* CMYK is most common 'worst case' */
 #define BITS_PER_CHANNEL 8      /* currently pdf14 device is always 8-bit */
 /* The estimated size of an individual PDF 1.4 buffer row (in bits) */
 #define ESTIMATED_PDF14_ROW_SIZE(width, target_num_components) ((width) * BITS_PER_CHANNEL\
-        * (NUM_ALPHA_CHANNELS + NUM_COLOR_CHANNELS))
+        * (NUM_ALPHA_CHANNELS + max(target_num_components,NUM_COLOR_CHANNELS)))
 /* The estimated size of one row in all PDF 1.4 buffers (in bits) */
 #define ESTIMATED_PDF14_ROW_SPACE(width, target_num_components) \
         (NUM_PDF14_BUFFERS * ESTIMATED_PDF14_ROW_SIZE(width, target_num_components))
