@@ -438,7 +438,12 @@ clist_init_data(gx_device * dev, byte * init_data, uint data_size)
                     pbdev->finalize(pbdev);
                 return_error(gs_error_rangecheck);
             }
-            bits_size = min(band_space - band_data_size, data_size >> 1);
+            /* If the tile_cache_size is specified, use it */
+            if (cdev->space_params.band.tile_cache_size == 0) {
+                bits_size = min(band_space - band_data_size, data_size >> 1);
+            } else {
+                bits_size = cdev->space_params.band.tile_cache_size;
+            }
         } else {
             int adjusted;
             /*
@@ -885,11 +890,13 @@ gx_color_index2usage(gx_device *dev, gx_color_index color)
     gx_color_usage_bits bits = 0;
     uchar i;
 
+    if (dev->color_info.polarity == GX_CINFO_POLARITY_ADDITIVE)
+        color = color ^ ~0;		/* white is 0 */
+
     for (i = 0; i < dev->color_info.num_components; i++) {
         if (color & dev->color_info.comp_mask[i])
             bits |= (1<<i);
     }
-
     return bits;
 }
 
@@ -1374,7 +1381,6 @@ clist_make_accum_device(gx_device *target, const char *dname, void *base, int sp
             float MediaSize[2];
             float ImagingBBox[4];
             bool  ImagingBBox_set;
-            float MarginsHWResolution[2];
             float Margins[2];
             float HWMargins[4];
             long  PageCount;
