@@ -298,6 +298,7 @@ gx_general_fill_path(gx_device * pdev, const gs_gstate * pgs,
     bool big_path = ppath->subpath_count > 50;
     fill_options fo;
     line_list lst;
+    int clipping = 0;
 
     *(const fill_options **)&lst.fo = &fo; /* break 'const'. */
     /*
@@ -355,6 +356,7 @@ gx_general_fill_path(gx_device * pdev, const gs_gstate * pgs,
             dev = (gx_device *) & cdev;
             gx_make_clip_device_on_stack(&cdev, pcpath, save_dev);
             cdev.max_fill_band = save_dev->max_fill_band;
+            clipping = 1;
         }
     }
     /*
@@ -585,6 +587,8 @@ gx_general_fill_path(gx_device * pdev, const gs_gstate * pgs,
                    stats_fill.slow_order);
     }
 #endif
+    if (clipping)
+        gx_destroy_clip_device_on_stack(&cdev);
     return code;
 }
 
@@ -1737,7 +1741,7 @@ intersect(active_line *endp, active_line *alp, fixed y, fixed y1, fixed *p_y_new
     fixed dx_old = alp->x_current - endp->x_current;
     fixed dx_den = dx_old + endp->x_next - alp->x_next;
 
-    if (dx_den <= dx_old)
+    if (dx_den <= dx_old || dx_den == 0)
         return false; /* Intersection isn't possible. */
     dy = y1 - y;
     if_debug3('F', "[F]cross: dy=%g, dx_old=%g, dx_new=%g\n",
@@ -1746,7 +1750,7 @@ intersect(active_line *endp, active_line *alp, fixed y, fixed y1, fixed *p_y_new
     /* Do the computation in single precision */
     /* if the values are small enough. */
     y_new =
-        ((dy | dx_old) < 1L << (size_of(fixed) * 4 - 1) ?
+        (((ufixed)(dy | dx_old)) < (1L << (size_of(fixed) * 4 - 1)) ?
          dy * dx_old / dx_den :
          (INCR_EXPR(mq_cross), fixed_mult_quo(dy, dx_old, dx_den)))
         + y;
