@@ -174,10 +174,12 @@ gx_image_enum_alloc(const gs_image_common_t * pic,
             )
             return_error(gs_error_rangecheck);
     }
+    *ppenum = NULL;		/* in case alloc fails and caller doesn't check code */
     penum = gs_alloc_struct(mem, gx_image_enum, &st_gx_image_enum,
                             "gx_default_begin_image");
     if (penum == 0)
         return_error(gs_error_VMerror);
+    memset(penum, 0, sizeof(gx_image_enum));	/* in case of failure, no dangling pointers */
     if (prect) {
         penum->rect.x = prect->p.x;
         penum->rect.y = prect->p.y;
@@ -288,7 +290,6 @@ gx_image_enum_begin(gx_device * dev, const gs_gstate * pgs,
     penum->Height = height;
 
     if ((code = gx_image_compute_mat(pgs, pmat, &(pim->ImageMatrix), &mat)) < 0) {
-        gs_free_object(mem, penum, "gx_default_begin_image");
         return code;
     }
     /* Grid fit: A common construction in postscript/PDF files is for images
@@ -575,6 +576,8 @@ gx_image_enum_begin(gx_device * dev, const gs_gstate * pgs,
            eliminate the 256 bytes for the >8bpp image enumerator */
         penum->clues = (gx_image_clue*) gs_alloc_bytes(mem, sizeof(gx_image_clue)*256,
                              "gx_image_enum_begin");
+        if (penum->clues == NULL)
+            return_error(gs_error_VMerror);
         penum->icolor0 = &(penum->clues[0].dev_color);
         penum->icolor1 = &(penum->clues[255].dev_color);
     } else {
@@ -583,7 +586,6 @@ gx_image_enum_begin(gx_device * dev, const gs_gstate * pgs,
     }
     if (masked) {       /* This is imagemask. */
         if (bps != 1 || pcs != NULL || penum->alpha || decode[0] == decode[1]) {
-            gs_free_object(mem, penum, "gx_default_begin_image");
             return_error(gs_error_rangecheck);
         }
         /* Initialize color entries 0 and 255. */
@@ -603,7 +605,6 @@ gx_image_enum_begin(gx_device * dev, const gs_gstate * pgs,
 
         spp = cs_num_components(pcs);
         if (spp < 0) {          /* Pattern not allowed */
-            gs_free_object(mem, penum, "gx_default_begin_image");
             return_error(gs_error_rangecheck);
         }
         if (penum->alpha)
@@ -711,7 +712,6 @@ gx_image_enum_begin(gx_device * dev, const gs_gstate * pgs,
     bsize = ((bps > 8 ? width * 2 : width) + 15) * spp;
     buffer = gs_alloc_bytes(mem, bsize, "image buffer");
     if (buffer == 0) {
-        gs_free_object(mem, penum, "gx_default_begin_image");
         return_error(gs_error_VMerror);
     }
     penum->bps = bps;
