@@ -34,10 +34,13 @@
 #endif
 
 /* The device descriptor */
+static dev_proc_open_device(bittag_open);
 static dev_proc_get_color_mapping_procs(bittag_get_color_mapping_procs);
 static dev_proc_map_rgb_color(bittag_rgb_map_rgb_color);
 static dev_proc_map_color_rgb(bittag_map_color_rgb);
 static dev_proc_put_params(bittag_put_params);
+static dev_proc_create_buf_device(bittag_create_buf_device);
+static dev_proc_fillpage(bittag_fillpage);
 static dev_proc_map_rgb_color(bit_mono_map_color);
 #if 0 /* unused */
 static dev_proc_map_rgb_color(bit_forcemono_map_rgb_color);
@@ -158,7 +161,7 @@ const gx_device_bit gs_bitcmyk_device =
 
 static const gx_device_procs bitrgbtags_procs =
     {
-        gdev_prn_open,                        /* open_device */
+        bittag_open,                        /* open_device */
         gx_default_get_initial_matrix,        /* initial_matrix */
         ((void *)0),                        /* sync_output */
         gdev_prn_output_page,                 /* output page */
@@ -219,7 +222,7 @@ static const gx_device_procs bitrgbtags_procs =
         ((void *)0),                        /* fill_linear_color_triangle */
         ((void *)0),                        /* update_spot_equivalent_colors */
         ((void *)0),                        /* ret_devn_params */
-        ((void *)0),                        /* fillpage */
+        bittag_fillpage,                    /* fillpage */
         ((void *)0),                        /* push_transparency_state */
         ((void *)0),                        /* pop_transparency_state */
         bit_put_image                        /* put_image */
@@ -283,7 +286,6 @@ const gx_device_bit gs_bitrgbtags_device =
         } ,
         0 ,
         { X_DPI, Y_DPI } ,
-        { X_DPI, Y_DPI },
         {(float)(-(0) * (X_DPI)),
          (float)(-(0) * (Y_DPI))},
         {(float)((0) * 72.0),
@@ -325,7 +327,7 @@ const gx_device_bit gs_bitrgbtags_device =
         { 0 },
         { bittags_print_page,
           gx_default_print_page_copies,
-          { gx_default_create_buf_device,
+          { bittag_create_buf_device,
             gx_default_size_buf_device,
             gx_default_setup_buf_device,
             gx_default_destroy_buf_device },
@@ -531,6 +533,40 @@ bit_map_cmyk_color(gx_device * dev, const gx_color_value cv[])
     return (color == gx_no_color_index ? color ^ 1 : color);
 }
 
+static int
+bittag_open(gx_device * pdev)
+{
+    gx_device_printer *ppdev = (gx_device_printer *)pdev;
+    int code;
+    /* We replace create_buf_device so we can replace copy_alpha
+     * for memory device, but not clist. We also have our own
+     * fillpage proc to fill with UNTOUCHED tag
+     */
+    ppdev->printer_procs.buf_procs.create_buf_device = bittag_create_buf_device;
+    code = gdev_prn_open(pdev);
+    return code;
+}
+
+/* fill the page fills with unmarked white */
+static int
+bittag_fillpage(gx_device *dev, gs_gstate * pgs, gx_device_color *pdevc)
+{
+    return (*dev_proc(dev, fill_rectangle))(dev, 0, 0, dev->width, dev->height,
+                                            GS_UNTOUCHED_TAG << 24 | 0xffffff);
+}
+
+static int
+bittag_create_buf_device(gx_device **pbdev, gx_device *target, int y,
+   const gx_render_plane_t *render_plane, gs_memory_t *mem,
+   gx_color_usage_t *color_usage)
+{
+    gx_device_printer *ptarget = (gx_device_printer *)target;
+    int code = gx_default_create_buf_device(pbdev, target, y,
+        render_plane, mem, color_usage);
+    set_dev_proc(*pbdev, fillpage, bittag_fillpage);
+    return code;
+}
+
 static int
 bittag_put_params(gx_device * pdev, gs_param_list * plist)
 {
@@ -760,15 +796,18 @@ bittags_print_page(gx_device_printer * pdev, FILE * prn_stream)
                  ((gx_device_bit *)pdev)->LastLine;
     int line_count = any_abs(bottom - lnum);
     int i, step = lnum > bottom ? -1 : 1;
+    int code = 0;
 
     if (in == 0)
         return_error(gs_error_VMerror);
 
-    fprintf(prn_stream, "P6\n%d %d\n255\n", pdev->width, pdev->height);
+    if (!nul)
+        fprintf(prn_stream, "P7\nWIDTH %d\nHEIGHT %d\nMAXVAL 255\nDEPTH 4\nTUPLTYPE RGB_TAG\nENDHDR\n", pdev->width, pdev->height);
     if ((lnum == 0) && (bottom == 0))
         line_count = pdev->height - 1;		/* default when LastLine == 0, FirstLine == 0 */
     for (i = 0; i <= line_count; i++, lnum += step) {
-        gdev_prn_get_bits(pdev, lnum, in, &data);
+        if ((code = gdev_prn_get_bits(pdev, lnum, in, &data)) < 0)
+            return code;
         if (!nul)
             fwrite(data, 1, line_size, prn_stream);
     }
@@ -777,9 +816,9 @@ bittags_print_page(gx_device_printer * pdev, FILE * prn_stream)
 }
 
 static int
-bit_put_image(gx_device *pdev, const byte *buffer, int num_chan, int xstart,
+bit_put_image(gx_device *pdev, const byte **buffers, int num_chan, int xstart,
               int ystart, int width, int height, int row_stride,
-              int plane_stride, int alpha_plane_index, int tag_plane_index)
+              int alpha_plane_index, int tag_plane_index)
 {
     gx_device_memory *pmemdev = (gx_device_memory *)pdev;
     byte *buffer_prn;
@@ -791,9 +830,6 @@ bit_put_image(gx_device *pdev, const byte *buffer, int num_chan, int xstart,
     if (alpha_plane_index != 0)
         return 0;                /* we don't want alpha, return 0 to ask for the    */
                                  /* pdf14 device to do the alpha composition        */
-    /* Eventually, the pdf14 device might be chunky pixels, punt for now */
-    if (plane_stride == 0)
-        return 0;
     if (num_chan != 3 || tag_plane_index <= 0)
             return_error(gs_error_unknownerror);        /* can't handle these cases */
     /* Drill down to get the appropriate memory buffer pointer */
@@ -805,11 +841,11 @@ bit_put_image(gx_device *pdev, const byte *buffer, int num_chan, int xstart,
         for ( x = xstart; x < xend; x++ ) {
             /* Tag data first, then RGB */
             buffer_prn[des_position] =
-                buffer[src_position + tag_plane_index * plane_stride];
+                buffers[tag_plane_index][src_position];
                 des_position += 1;
             for ( k = 0; k < 3; k++) {
                 buffer_prn[des_position] =
-                    buffer[src_position + k * plane_stride];
+                    buffers[k][src_position];
                     des_position += 1;
             }
             src_position += 1;
