@@ -21,6 +21,7 @@
 #include "strimpl.h"
 #include "sdct.h"
 #include "sjpeg.h"
+#include "gxdownscale.h"
 
 /* Structure for the JPEG-writing device. */
 typedef struct gx_device_jpeg_s {
@@ -39,6 +40,7 @@ typedef struct gx_device_jpeg_s {
      */
     gs_point ViewTrans;
 
+    gx_downscaler_params downscale;
 } gx_device_jpeg;
 
 /* The device descriptor */
@@ -93,7 +95,8 @@ const gx_device_jpeg gs_jpeg_device =
  0,				/* JPEGQ: 0 indicates not specified */
  0.0,				/* QFactor: 0 indicates not specified */
  { 1.0, 1.0 },                  /* ViewScale 1 to 1 */
- { 0.0, 0.0 }                   /* translation 0 */
+ { 0.0, 0.0 },                  /* translation 0 */
+ GX_DOWNSCALER_PARAMS_DEFAULTS
 };
 
 /* 8-bit gray */
@@ -132,7 +135,8 @@ const gx_device_jpeg gs_jpeggray_device =
  0,				/* JPEGQ: 0 indicates not specified */
  0.0,				/* QFactor: 0 indicates not specified */
  { 1.0, 1.0 },                  /* ViewScale 1 to 1 */
- { 0.0, 0.0 }                   /* translation 0 */
+ { 0.0, 0.0 },                   /* translation 0 */
+ GX_DOWNSCALER_PARAMS_DEFAULTS
 };
 /* 32-bit CMYK */
 
@@ -167,7 +171,8 @@ const gx_device_jpeg gs_jpegcmyk_device =
  0,				/* JPEGQ: 0 indicates not specified */
  0.0,				/* QFactor: 0 indicates not specified */
  { 1.0, 1.0 },                  /* ViewScale 1 to 1 */
- { 0.0, 0.0 }                   /* translation 0 */
+ { 0.0, 0.0 },                   /* translation 0 */
+ GX_DOWNSCALER_PARAMS_DEFAULTS
 };
 
 /* Apparently Adobe Photoshop and some other applications that	*/
@@ -211,6 +216,9 @@ jpeg_get_params(gx_device * dev, gs_param_list * plist)
     if (code < 0)
         return code;
 
+    ecode = 0;
+    if ((code = gx_downscaler_write_params(plist, &jdev->downscale, 0)) < 0)
+        ecode = code;
     if ((ecode = param_write_int(plist, "JPEGQ", &jdev->JPEGQ)) < 0)
         code = ecode;
     if ((ecode = param_write_float(plist, "QFactor", &jdev->QFactor)) < 0)
@@ -243,6 +251,8 @@ jpeg_put_params(gx_device * dev, gs_param_list * plist)
     float qf = jdev->QFactor;
     float fparam;
 
+    ecode = gx_downscaler_read_params(plist, &jdev->downscale, 0);
+
     switch (code = param_read_int(plist, (param_name = "JPEGQ"), &jq)) {
         case 0:
             if (jq < 0 || jq > 100)
@@ -420,11 +430,17 @@ jpeg_print_page(gx_device_printer * pdev, FILE * prn_stream)
     int code;
     stream_DCT_state state;
     stream fstrm, jstrm;
+    gx_downscaler_t ds;
 
     if (jcdp == 0 || in == 0) {
         code = gs_note_error(gs_error_VMerror);
         goto fail;
     }
+    code = gx_downscaler_init(&ds, (gx_device *)jdev, 8, 8,
+                              jdev->color_info.depth/8, jdev->downscale.downscale_factor, 0, NULL, 0);
+    if (code < 0)
+        goto done;
+
     /* Create the DCT encoder state. */
     jcdp->templat = s_DCTE_template;
     s_init_state((stream_state *)&state, &jcdp->templat, 0);
@@ -449,17 +465,20 @@ jpeg_print_page(gx_device_printer * pdev, FILE * prn_stream)
             !(pdev->icc_struct->usefastcolor)) {
             state.icc_profile = icc_profile;
         }
-    } 
+    }
     /* We need state.memory for gs_jpeg_create_compress().... */
     jcdp->memory = state.jpeg_memory = state.memory = mem;
     if ((code = gs_jpeg_create_compress(&state)) < 0)
+    {
+        gx_downscaler_fin(&ds);
         goto fail;
+    }
     /* ....but we need it to be NULL so we don't try to free
      * the stack based state...
      */
     state.memory = NULL;
-    jcdp->cinfo.image_width = pdev->width;
-    jcdp->cinfo.image_height = pdev->height;
+    jcdp->cinfo.image_width = gx_downscaler_scale(pdev->width, jdev->downscale.downscale_factor);
+    jcdp->cinfo.image_height = gx_downscaler_scale(pdev->height, jdev->downscale.downscale_factor);
     switch (pdev->color_info.depth) {
         case 32:
             jcdp->cinfo.input_components = 4;
@@ -524,7 +543,7 @@ jpeg_print_page(gx_device_printer * pdev, FILE * prn_stream)
         (*state.templat->init) (jstrm.state);
 
     /* Copy the data to the output. */
-    for (lnum = 0; lnum < pdev->height; ++lnum) {
+    for (lnum = 0; lnum < jcdp->cinfo.image_height; ++lnum) {
         byte *data;
         uint ignore_used;
 
@@ -532,8 +551,8 @@ jpeg_print_page(gx_device_printer * pdev, FILE * prn_stream)
             code = gs_note_error(gs_error_ioerror);
             goto done;
         }
-        gdev_prn_get_bits(pdev, lnum, in, &data);
-        sputs(&jstrm, data, state.scan_line_size, &ignore_used);
+        gx_downscaler_getbits(&ds, in, lnum);
+        sputs(&jstrm, in, state.scan_line_size, &ignore_used);
     }
 
     /* Wrap up. */
@@ -545,6 +564,7 @@ jpeg_print_page(gx_device_printer * pdev, FILE * prn_stream)
     gs_free_object(mem, fbuf, "jpeg_print_page(fbuf)");
     if (jcdp)
         gs_jpeg_destroy(&state);	/* frees *jcdp */
+    gx_downscaler_fin(&ds);
     gs_free_object(mem, in, "jpeg_print_page(in)");
     return code;
   fail:
