@@ -214,9 +214,13 @@ pdf_copy_mono(gx_device_pdf *pdev,
         set_image_color(pdev, zero);
     } else if (zero == pdev->black && one == pdev->white) {
         pcs = gs_cspace_new_DeviceGray(pdev->memory);
+        if (pcs == NULL)
+            return_error(gs_error_VMerror);
         gs_image_t_init(&image, pcs);
     } else if (zero == pdev->white && one == pdev->black) {
         pcs = gs_cspace_new_DeviceGray(pdev->memory);
+        if (pcs == NULL)
+            return_error(gs_error_VMerror);
         gs_image_t_init(&image, pcs);
         invert = 0xff;
     } else {
@@ -313,13 +317,19 @@ pdf_copy_mono(gx_device_pdf *pdev,
         psdf_setup_image_filters((gx_device_psdf *) pdev, &writer.binary[0],
                                  (gs_pixel_image_t *)&image, NULL, NULL, true, in_line);
     }
-    pdf_begin_image_data(pdev, &writer, (const gs_pixel_image_t *)&image,
+    code = pdf_begin_image_data(pdev, &writer, (const gs_pixel_image_t *)&image,
                          pcsvalue, 0);
+    if (code < 0)
+        return code;
+
     code = pdf_copy_mask_bits(writer.binary[0].strm, base, sourcex, raster,
                               w, h, invert);
     if (code < 0)
         return code;
-    pdf_end_image_binary(pdev, &writer, writer.height);
+    code = pdf_end_image_binary(pdev, &writer, writer.height);
+    if (code < 0)
+        return code;
+
     if (!pres) {
         switch ((code = pdf_end_write_image(pdev, &writer))) {
             default:                /* error */
