@@ -101,10 +101,11 @@ pdf_add_ToUnicode(gx_device_pdf *pdev, gs_font *font, pdf_font_resource_t *pdfon
             char *d2 = strchr(hexdigits, gnstr->data[5]);
             char *d3 = strchr(hexdigits, gnstr->data[6]);
 
-            unicode = (ushort *)gs_alloc_bytes(pdev->memory, 2 * sizeof(ushort), "temporary Unicode array");
+            unicode = (ushort *)gs_alloc_bytes(pdev->memory, sizeof(ushort), "temporary Unicode array");
             if (d0 != NULL && d1 != NULL && d2 != NULL && d3 != NULL) {
-                unicode[0] = ((d0 - hexdigits) << 8) + ((d1 - hexdigits));
-                unicode[1] = ((d2 - hexdigits) << 8) + ((d3 - hexdigits));
+                char *u = (char *)unicode;
+                u[0] = ((d0 - hexdigits) << 4) + ((d1 - hexdigits));
+                u[1] = ((d2 - hexdigits) << 4) + ((d3 - hexdigits));
                 length = 2;
             }
         }
@@ -443,10 +444,10 @@ process_text_estimate_bbox(pdf_text_enum_t *pte, gs_font_base *font,
         gs_point_transform(font->FontBBox.p.x, font->FontBBox.q.y, &m, &p1);
         gs_point_transform(font->FontBBox.q.x, font->FontBBox.p.y, &m, &p2);
         gs_point_transform(font->FontBBox.q.x, font->FontBBox.q.y, &m, &p3);
-        bbox.p.x = min(min(p0.x, p1.x), min(p1.x, p2.x)) + total.x;
-        bbox.p.y = min(min(p0.y, p1.y), min(p1.y, p2.y)) + total.y;
-        bbox.q.x = max(max(p0.x, p1.x), max(p1.x, p2.x)) + total.x;
-        bbox.q.y = max(max(p0.y, p1.y), max(p1.y, p2.y)) + total.y;
+        bbox.p.x = min(min(p0.x, p1.x), min(p2.x, p3.x)) + total.x;
+        bbox.p.y = min(min(p0.y, p1.y), min(p2.y, p3.y)) + total.y;
+        bbox.q.x = max(max(p0.x, p1.x), max(p2.x, p3.x)) + total.x;
+        bbox.q.y = max(max(p0.y, p1.y), max(p2.y, p3.y)) + total.y;
         if (i == 0)
             *text_bbox = bbox;
         else
