@@ -57,6 +57,7 @@
 #include "ivmspace.h"
 #include "idisp.h"              /* for setting display device callback */
 #include "iplugin.h"
+#include "zfile.h"
 
 #ifdef PACIFY_VALGRIND
 #include "valgrind.h"
@@ -136,7 +137,6 @@ int
 gs_main_init0(gs_main_instance * minst, FILE * in, FILE * out, FILE * err,
               int max_lib_paths)
 {
-    ref *paths;
     ref *array;
 
     /* Do platform-dependent initialization. */
@@ -159,12 +159,6 @@ gs_main_init0(gs_main_instance * minst, FILE * in, FILE * out, FILE * err,
     gp_get_realtime(minst->base_time);
 
     /* Initialize the file search paths. */
-    paths = (ref *) gs_alloc_byte_array(minst->heap, max_lib_paths, sizeof(ref),
-                                        "lib_path array");
-    if (paths == 0) {
-        gs_lib_finit(1, gs_error_VMerror, minst->heap);
-        return_error(gs_error_VMerror);
-    }
     array = (ref *) gs_alloc_byte_array(minst->heap, max_lib_paths, sizeof(ref),
                                         "lib_path array");
     if (array == 0) {
@@ -187,22 +181,23 @@ gs_main_init0(gs_main_instance * minst, FILE * in, FILE * out, FILE * err,
 int
 gs_main_init1(gs_main_instance * minst)
 {
+    gs_dual_memory_t idmem;
+    name_table *nt = NULL;
+    int code = 0;
+
     if (minst->init_done < 1) {
-        gs_dual_memory_t idmem;
-        int code =
-            ialloc_init(&idmem, minst->heap,
+        code = ialloc_init(&idmem, minst->heap,
                         minst->memory_clump_size, gs_have_level2());
 
         if (code < 0)
             return code;
         code = gs_lib_init1((gs_memory_t *)idmem.space_system);
         if (code < 0)
-            return code;
+            goto fail;
         alloc_save_init(&idmem);
         {
             gs_memory_t *mem = (gs_memory_t *)idmem.space_system;
-            name_table *nt = names_init(minst->name_table_size,
-                                        idmem.space_system);
+            nt = names_init(minst->name_table_size, idmem.space_system);
 
             if (nt == 0)
                 return_error(gs_error_VMerror);
@@ -211,20 +206,25 @@ gs_main_init1(gs_main_instance * minst)
                                            (void **)&mem->gs_lib_ctx->gs_name_table,
                                            "the_gs_name_table");
             if (code < 0)
-                return code;
+                goto fail;
+            mem->gs_lib_ctx->client_check_file_permission = z_check_file_permissions;
         }
         code = obj_init(&minst->i_ctx_p, &idmem);  /* requires name_init */
         if (code < 0)
-            return code;
+            goto fail;
         code = i_plugin_init(minst->i_ctx_p);
         if (code < 0)
-            return code;
+            goto fail;
         code = i_iodev_init(minst->i_ctx_p);
         if (code < 0)
-            return code;
+            goto fail;
         minst->init_done = 1;
     }
     return 0;
+
+fail:
+    names_free(nt);
+    return code;
 }
 
 /*
@@ -854,6 +854,21 @@ static char *gs_main_tempnames(gs_main_instance *minst)
     return tempnames;
 }
 
+static void
+gs_finit_push_systemdict(i_ctx_t *i_ctx_p)
+{
+    if (i_ctx_p == NULL)
+        return;
+    if (dsp == dstop ) {
+        if (ref_stack_extend(&d_stack, 1) < 0) {
+            /* zend() cannot fail */
+            (void)zend(i_ctx_p);
+        }
+    }
+    dsp++;
+    ref_assign(dsp, systemdict);
+}
+
 /* Free all resources and return. */
 int
 gs_main_finit(gs_main_instance * minst, int exit_status, int code)
@@ -874,6 +889,13 @@ gs_main_finit(gs_main_instance * minst, int exit_status, int code)
      */
     tempnames = gs_main_tempnames(minst);
 
+    /* by the time we get here, we *must* avoid any random redefinitions of
+     * operators etc, so we push systemdict onto the top of the dict stack.
+     * We do this in C to avoid running into any other re-defininitions in the
+     * Postscript world.
+     */
+    gs_finit_push_systemdict(i_ctx_p);
+
 #ifndef PSI_INCLUDED
     /* We have to disable BGPrint before we call interp_reclaim() to prevent the
      * parent rendering thread initialising for the next page, whilst we are
@@ -881,13 +903,12 @@ gs_main_finit(gs_main_instance * minst, int exit_status, int code)
      * We also have to mess with the BeginPage/EndPage procs so that we don't
      * trigger a spurious extra page to be emitted.
      */
-    if (minst->init_done >= 1) {
+    if (minst->init_done >= 2) {
         gs_main_run_string(minst,
-            "/systemdict .systemexec /begin .systemexec \
-            /BGPrint /GetDeviceParam .special_op \
+            "/BGPrint /GetDeviceParam .special_op \
             {{ <</BeginPage {pop} /EndPage {pop pop //false } \
               /BGPrint false /NumRenderingThreads 0>> setpagedevice} if} if \
-              serverdict /.jobsavelevel get 0 eq {/quit} {/stop} ifelse end \
+              serverdict /.jobsavelevel get 0 eq {/quit} {/stop} ifelse \
               .systemvar exec",
             0 , &exit_code, &error_object);
     }
@@ -897,7 +918,7 @@ gs_main_finit(gs_main_instance * minst, int exit_status, int code)
      * Close the "main" device, because it may need to write out
      * data before destruction. pdfwrite needs so.
      */
-    if (minst->init_done >= 1) {
+    if (minst->init_done >= 2) {
         int code = 0;
 
         if (idmemory->reclaim != 0) {
@@ -952,16 +973,13 @@ gs_main_finit(gs_main_instance * minst, int exit_status, int code)
                 exit_status = code;
         }
 #endif
-    }
-    /* Flush stdout and stderr */
-    if (minst->init_done >= 2)
+      /* Flush stdout and stderr */
       gs_main_run_string(minst,
         "(%stdout) (w) file closefile (%stderr) (w) file closefile \
-        /systemdict .systemexec /begin .systemexec \
-          serverdict /.jobsavelevel get 0 eq {/quit} {/stop} ifelse .systemexec \
-          systemdict /savedinitialgstate .forceundef \
-        end ",
+        serverdict /.jobsavelevel get 0 eq {/quit} {/stop} ifelse .systemexec \
+          systemdict /savedinitialgstate .forceundef",
         0 , &exit_code, &error_object);
+    }
     gp_readline_finit(minst->readline_data);
     i_ctx_p = minst->i_ctx_p;		/* get current interp context */
     if (gs_debug_c(':')) {
@@ -1003,6 +1021,7 @@ gs_main_finit(gs_main_instance * minst, int exit_status, int code)
 #ifndef PSI_INCLUDED
     gs_lib_finit(exit_status, code, minst->heap);
 #endif
+    gs_free_object(minst->heap, minst->lib_path.container.value.refs, "lib_path array");
     return exit_status;
 }
 int
